# -*- coding: utf-8 -*-
"""
2024-04-17 14:34:26
Kai Li
Institutes of Environmental Sciences
Leiden University
Please cite the reseach article related to the provided code. 
For any questions, please contact k.li@cml.leidenuniv.nl
"""

#%% 1. Import packages
import pandas as pd
import copy
import numpy as np
import time
import itertools
import comtradeapicall                                                         ## UN Comtrade API Package https://github.com/uncomtrade/comtradeapicall
import pickle
import csv
import json
from urllib.request import urlopen
import matplotlib.pyplot as plt
import matplotlib
import plotly.graph_objects as go
import plotly.io as io
from plotly.subplots import make_subplots
#%% 2. Functions and lists defined

## get_key1(val)
dict_cn = {"Southeast Asia":["Malaysia", "Lao People's Dem. Rep.", "Thailand",
                               "Indonesia", "Viet Nam", "Philippines"],

              "Other Asia": ["China","China, Hong Kong SAR","China, Macao SAR",
                            "Japan","Mongolia", "Rep. of Korea",
                            "Dem. People's Rep. of Korea", "Other Asia, nes", 
                            "India", "Pakistan", "Uzbekistan", "Saudi Arabia",
                            "Yemen", "Afghanistan", "Oman", "Singapore",
                            "Sri Lanka", "United Arab Emirates", "Israel",
                            "Myanmar", "Iran", "Lebanon", "Kazakhstan",
                            "Kyrgyzstan", "State of Palestine", "Tajikistan",
                            "Bahrain", "Cambodia", "Brunei Darussalam",
                            "Qatar", "Jordan", "Armenia", "Kuwait", "Nepal",
                            "Maldives", "Bangladesh","Türkiye", "Turkey",
                            "Timor-Leste", "Iraq", "Syria"],
              "Western Europe": ["Netherlands", "Germany", "Belgium", "France",
                                 "Luxembourg", "Denmark", "Switzerland",
                                 "Ireland", "Sweden","United Kingdom",
                                 "Austria", "Italy","Spain","Portugal",
                                 "Norway", "Finland","Iceland"],
              "Eastern Europe": ["Poland", "Czechia","Slovenia", "Romania",
                                 "Ukraine", "Lithuania", "Bulgaria",
                                 "Russian Federation", "Croatia", "Greece",
                                 "Latvia", "Serbia", "Slovakia", "Hungary",
                                 "Belarus", "Azerbaijan", "Bosnia Herzegovina",
                                 "North Macedonia", "Estonia", "Georgia",
                                 "Montenegro", "Cyprus", "Rep. of Moldova",
                                 "Malta", "Andorra", "Albania"],
              "North America": ["USA", "Canada", "Bahamas","El Salvador", 
                                "Costa Rica", "Dominican Rep.", "Guatemala",
                                "Nicaragua", "Honduras",  "Belize", "Jamaica", 
                                "Saint Lucia", "Aruba", "Barbados", "Bermuda",
                                "Grenada", "Antigua and Barbuda", "Mexico", 
                                "Panama","Saint Vincent and the Grenadines"],
              
              "South America": ["Peru","Colombia","Ecuador","Chile","Brazil", 
                                "Paraguay","Bolivia (Plurinational State of)",
                                "Uruguay", "Guyana","Argentina","Cayman Isds",
                                "Saint Kitts and Nevis", "Suriname", 
                                "Trinidad and Tobago","Br. Virgin Isds", 
                                "Cuba", "Haiti","Venezuela", "Dominica", 
                                "Curaçao"],
           "Africa": ["Nigeria", "Egypt", "South Africa", "Morocco", "Tunisia",
                       "Ghana", "Senegal", "United Rep. of Tanzania", "Kenya",
                       "Uganda", "Zimbabwe", "Ethiopia", "Botswana", "Togo",
                       "Mozambique", "Angola", "Sierra Leone",
                       "Dem. Rep. of the Congo", "Rwanda", "Malawi", "Namibia",
                       "Mali", "Sudan", "Mauritania", "Burkina Faso",
                       "Cameroon", "Eswatini", "Gambia", "Madagascar", "Zambia",
                       "Comoros", "Mauritius", "Sao Tome and Principe",
                       "Lesotho", "Niger", "Benin", "Burundi", "Seychelles",
                       "Cabo Verde", "Congo", "Somalia", "Guinea", "Djibouti",
                       "Côte d'Ivoire", "Liberia", "Algeria"],
            "Oceania": ["Australia", "New Zealand", "Solomon Isds", "Fiji",
                        "French Polynesia", "Palau", "Samoa", "Kiribati",
                        "Cocos Isds"]}


dict7 = {"EU":["Netherlands", "Germany", "Belgium", "France",
                   "Portugal", "Switzerland", "Italy",
                    "Spain", "Czechia","Slovenia","Poland","Austria"],

              "intra-Asia": ["Türkiye","China, Hong Kong SAR", "Japan", 
                             "Rep. of Korea","Other Asia, nes", "India",
                             'Malaysia', 'Thailand', 'Indonesia', 'Viet Nam', 
                             'Philippines',"China"],

              "intra-America": ["USA", "Canada", "Mexico"],
              'UK':['United Kingdom']}

dict8 = {'non-OECD':['Thailand', 'Malaysia', 'Other Asia, nes', 'Indonesia',
                   'Viet Nam', 'China, Hong Kong SAR', 'India','China',
                   'Philippines'],

              'OECD': ['Germany', 'Canada', 'Spain', 'Italy', 'Portugal',
                       'Japan', 'Slovenia', 'Czechia','Türkiye', 'Netherlands', 
                       'Mexico', 'United Kingdom', 'Belgium','France',
                       'Switzerland','Rep. of Korea','Poland','Austria','USA']}

dict9 = {'mismanage':['Thailand', 'Malaysia', 'Indonesia', 'Viet Nam', 'India',
                      'China','Türkiye'],

              'non-mismanage': ['Germany', 'Canada', 'Spain', 'Italy', 'Portugal',
                       'Japan', 'Slovenia', 'Czechia', 'Netherlands', 
                       'Mexico', 'United Kingdom', 'Belgium','France',
                       'Switzerland','Rep. of Korea','Poland','Austria','USA']}

dict10 = {"Other Europe exporter":["Belgium", "France", "Italy", "Spain", "Poland","Austria"],
          "Other Asia exporter": ["Türkiye","Other Asia, nes", 'Malaysia', 'Indonesia', 'Viet Nam' ],
          "Japan":['Japan'],
          'Netherlands':['Netherlands'],
          'Germany':['Germany'],
          'United Kingdom':['United Kingdom'],
          "North America":['USA','Canada','Mexico']}

dict11 = {"Other Europe importer":['United Kingdom', "Belgium", "France", "Italy", 
                          "Spain", "Poland","Austria"],
          "Other Asia importer": ['Japan',"Other Asia, nes"],
          'Malaysia': ['Malaysia'],
          'Indonesia': ['Indonesia'], 
          'Viet Nam' : ['Viet Nam'],
          "Türkiye": ['Türkiye'],
          'Netherlands':['Netherlands'],
          'Germany':['Germany'],          
          "North America":['USA','Canada','Mexico']}

## Extracting continents(key)
def get_key7(val):
     for key, value in dict7.items():
         if val in value:
             return key
     return np.nan


## Extracting continents(key)
def get_key(val):
     for key, value in dict_cn.items():
         if val in value:
             return key
         
def get_key8(val):
     for key, value in dict8.items():
         if val in value:
             return key
     return np.nan
 
def get_key9(val):
     for key, value in dict9.items():
         if val in value:
             return key
     return np.nan    

def get_key10(val):
     for key, value in dict10.items():
         if val in value:
             return key
     return np.nan 
 
def get_key11(val):
     for key, value in dict11.items():
         if val in value:
             return key
     return np.nan 
 
lst_ISO3 = ["DEU", "CAN", "ESP", "SVN", "THA", "CZE", "MYS", "TUR", "TWN", 
        "IDN", "VNM", "NLD", "MEX", "GBR", "HKG", "BEL", "FRA", "IND", 
        "CHN", "CHE", "KOR", "ITA", "PHL", "PRT", "USA", "JPN"]

lst_lc = ['Germany', 'Canada', 'Spain', 'Slovenia', 'Thailand', 'Czechia',
          'Malaysia', 'Türkiye', 'Other Asia, nes', 'Indonesia', 'Viet Nam',
          'Netherlands', 'Mexico', 'United Kingdom', 'China, Hong Kong SAR',
          'Belgium', 'France', 'India', 'China', 'Switzerland', 'Rep. of Korea',
          'Italy', 'Philippines', 'Portugal',  'USA', 'Japan']

dict_5 = dict(zip(lst_ISO3, lst_lc))


def get_key5(val):
     for key, value in dict_5.items():
         if val in value:
             return key 
         
lst_mot = ["Sea","Water","Air","Land","Railway","Road",
                          "Postal consignments, mail or courier shipment",
                          "Self propelled goods", "Other MOT"]  

lst_omot =  ["TOTAL MOT", "Other MOT", "Self propelled goods",
             "Postal consignments, mail or courier shipment", 
             "Pipelines and cables", "Inland waterway"]     
     
         
#%% 3. Retrieve bilateral trade of 2013-2022 from the UN Comtrade database 


df = comtradeapicall.getFinalData(subscription_key = 
     "xxx", typeCode='C', freqCode='A',                                        ## Please replace to your key for API of UN Comtrade
     clCode='HS', period= '2013,2014,2015,2016,2017,2018,2019,2020,2021,2022', 
     reporterCode=None,cmdCode='391510,391520,391530,391590', flowCode='M,X', 
      partnerCode=None, partner2Code=0, customsCode='C00',
      motCode=None, format_output='JSON', aggregateBy=None, 
     breakdownMode='plus', countOnly=None, includeDesc=True)                   
                                       
print(f'The update data is {df.shape},'
      f' {time.strftime("%Y-%m-%d %H:%M:%S")}')                                ## The update data is (137731, 47), 2023-10-02 12:11:55

df.to_pickle("./model/pkl/df10.pkl")    
                    
#%% 4. Read original data 2013-2022

df = pd.read_pickle("./model/pkl/df10.pkl")                                
lst_o = []
for i in range(2013, 2023):
    ## i = 2015
    data = df[df.period == f'{i}']
    data =data[data['partnerDesc'] != "World"].reset_index(drop = True) 
    lst_o.append(data)
    
#%% 5. Drop "TOTAL MOT" in the reporter country if other MOT exist     
start_time = time.time()
lst2 = copy.deepcopy(lst_o)
for t in lst2:
    ## t = lst2[0]
    lst = list(set(t["reporterDesc"]).union(set(t["partnerDesc"])))
    for i in itertools.product(lst,repeat=2):                                  ## cartesian product, equivalent to a nested for-loop

        df_m = t[(t['reporterDesc'] == i[0])&
                 (t['partnerDesc'] == i[1])&(t['flowCode'] == "M")]

        df_x = t[(t['reporterDesc'] == i[0])&
                 (t['partnerDesc'] == i[1])&(t['flowCode'] == "X")]

        if any(k in df_m["motDesc"].tolist() for k in lst_mot):
            t.drop(df_m[df_m["motDesc"] == "TOTAL MOT"].index, inplace = True)
        if any(k in df_x["motDesc"].tolist() for k in lst_mot):
            t.drop(df_x[df_x["motDesc"] == "TOTAL MOT"].index, inplace = True) 
end_time = time.time()
time_taken ='{:.2f}'.format((end_time - start_time)/60)                        ## Time taken: 19.50mins
print(f"Time taken: {time_taken} mins {time.strftime('%Y-%m-%d %H:%M:%S')}")            
            
with open('/model/pkl/lst2_model.pkl', 'wb') as f:
    pickle.dump(lst2, f)                                                       
with open('/model/pkl/lst2_model.pkl', 'rb') as f:
    lst2 = pickle.load(f)                                                      
    
#%% 6. Replace 0 or nan netweight if other weight metrics(altQty or qty) exist

lst3 = copy.deepcopy(lst2)

lst = [len(s[s["netWgt"].isin([0, np.nan])]) for s in lst3]
print(f"{lst} zero or nan netWgts in lst3,"
      f' {time.strftime("%Y-%m-%d %H:%M:%S")}')                                ## [23, 36, 29, 37, 62, 39, 71, 64, 60, 58] zero or nan netWgts in lst3, 2023-10-02 14:56:01

for k in lst3:
    for i in k.index:
        ##i= 2254
        t= k['netWgt'][i]
        if pd.isnull(t) or t==0:
            if pd.isnull(k['altQty'][i]) is False and k['altQty'][i]!=0:
               k.loc[i,'netWgt'] = k['altQty'][i]
            if pd.isnull(k['qty'][i]) is False and k['qty'][i]!=0:
               k.loc[i,'netWgt'] = k['qty'][i]
    
lst = [len(s[s["netWgt"].isin([0, np.nan])]) for s in lst3]
print(f"{lst} zero or nan netWgts in lst3,"
      f' {time.strftime("%Y-%m-%d %H:%M:%S")}')                                ## [6, 17, 6, 20, 6, 3, 10, 0, 0, 0] zero or nan netWgts in lst3, 2023-10-02 14:56:02

#%% 7. Replace zero and empty netweight values from similar trade

lst4 = copy.deepcopy(lst3)
lst_s, lst_r = [],[]
lst_c = copy.deepcopy(lst_o)                                                     
df_lst = pd.concat(lst_c).reset_index(drop = True) 

start_time = time.time()
for r,c in zip(lst4,lst_c):      
    for i in r.index : 
        rn = r.loc[i,"netWgt"]
        if pd.isna(rn) or rn == 0:              
           m, n, k = r["reporterDesc"], r["partnerDesc"], r["motDesc"]
           m1, n1, k1 = c["reporterDesc"], c["partnerDesc"], c["motDesc"]            
           m2, n2 = df_lst["reporterDesc"], df_lst["partnerDesc"]
           
           df_v6 = pd.concat([c[(m1 == m[i])&(n1 == n[i])&(k1 == k[i])],
                 c[(m1 == n[i])&(n1 == m[i])&(k1 == k[i])]]).drop_duplicates() ## select the similar trade (same trade year, countries, MOT) from lst_v2
                              
           df_v61 = pd.concat([c[(m1 == m[i])&(n1 == n[i])],
                              c[(m1 == n[i])&(n1 == m[i])]]).drop_duplicates() ## same trade year, countries
                               
           df_v62 = pd.concat([df_lst[(m2 == m[i])&(n2 == n[i])], 
                        df_lst[(m2 == n[i])&(n2 == m[i])]]).drop_duplicates()  ## same countries only (years from 2018 to 2021)
                               
           cc = 0
           tv = r["primaryValue"][i]                                           ## make sure the primaryValue is non-zero
           if tv != 0 and len(df_v6) >= 1 and\
               any(x > 0 for x in df_v6["netWgt"]):
              lp, s = 0, 0
              for j in df_v6.index:
                  if df_v6["netWgt"][j] > 0:
                     dn = df_v6["primaryValue"][j]/df_v6["netWgt"][j]          ## use netWgts from other similar trade to divide primaryValue
                     lp = lp + 1
                     s = s + dn
              v6_nw = s/lp
              r.loc[i, "netWgt"] = tv/v6_nw
              lst_s.append(r.loc[[i]]) 
              cc = 1                  
           if tv != 0 and cc == 0 and len(df_v61) >= 1 and\
               any(x > 0 for x in df_v61["netWgt"]):
              lp, s = 0, 0
              for j in df_v61.index:                      
                  if df_v61["netWgt"][j] > 0:
                     dn = df_v61["primaryValue"][j]/df_v61["netWgt"][j]
                     lp = lp + 1
                     s = s + dn
              v6_nw = s/lp
              r.loc[i, "netWgt"] = tv/v6_nw
              lst_s.append(r.loc[[i]])
              cc = 1
           if tv != 0 and cc == 0 and len(df_v62) >= 1 and\
               any(x > 0 for x in df_v62["netWgt"]):                       
              lp, s = 0, 0
              for j in df_v62.index:
                   if df_v62["netWgt"][j] > 0:
                      dn = df_v62["primaryValue"][j]/df_v62["netWgt"][j]
                      lp = lp + 1
                      s = s + dn
              v6_nw = s/lp
              r.loc[i, "netWgt"] = tv/v6_nw
              lst_s.append(r.loc[[i]])
              cc = 1
           if cc == 0:
              lst_r.append(r.loc[[i]])     
           
df_v6r = pd.concat(lst_s)    

print(f"After substitutes from the similar trade {len(lst_s)} "
      f"zero netWgts are replaced, {len(lst_r)} zero or empty values"
      " are left. {time.strftime('%Y-%m-%d %H:%M:%S')}")
                                                                               ## After substitutes from the similar trade 66 zero netWgts are replaced, 2 zero or empty values are left.
                                                                               
end_time = time.time()
time_taken = '{:.2f}'.format((end_time - start_time)/60)                       ##Time taken: 0.11 mins
print(f"Time taken: {time_taken} mins")  

#%% 8. Average only: drop "TOTAL MOT" and repeated trade from the partner 

lst5 = copy.deepcopy(lst4)                                        
lst_om = []
mot_s, mot_l= {"Water","Sea"}, {"land", "Road"}
lst_sum = []

start_time = time.time()
for t in lst5:
    p, q, c, d = t["reporterDesc"], t["partnerDesc"], t["cmdCode"], t["flowCode"]    
    lst_iter = list(set(t["reporterDesc"]).union(set(t["partnerDesc"])))
    for i in itertools.permutations(lst_iter,2):                               ## r-length tuples, all possible orderings, no repeated elements, ordered combinations. https://docs.python.org/3/library/itertools.html
        ## i = ('Netherlands', 'Germany')
        for k in ["391510", "391520", "391530", "391590"]:
            # k ="391590"
            m = t.loc[(c == k)&(d == "X")&(p == i[0])&(q == i[1])]
            n = t.loc[(c == k)&(d == "M")&(p == i[1])&(q == i[0])]
            mv, nv = m["netWgt"].values, n["netWgt"].values
            mm, nm = m["motDesc"].tolist(), n["motDesc"].tolist()
            df_mn = pd.concat([m,n])

            if len(m) == len(n) == 1:  ## 1
               cp_v7 = 0
               uxy = set(mm).union(set(nm))

               if mm == nm or uxy == mot_s or uxy == mot_l:                    ## if they use the same transport
                  t.loc[n.index,"netWgt"] = (mv[0] + nv[0])/2
                  t.drop(m.index,inplace = True)
                  cp_v7 = 1

               if bool(uxy & set(["TOTAL MOT", 'Other MOT'])) and cp_v7 == 0:  ## if one side or both sides have unspecified transport
                   if uxy == {"TOTAL MOT", "Other MOT"}:
                      t.loc[n.index, "netWgt"] = (mv[0] + nv[0])/2
                      t.drop(m.index,inplace = True)
                      cp_v7 = 1
                   else:
                      a = df_mn.index[~df_mn["motDesc"].isin(["TOTAL MOT",
                                                                'Other MOT'])]
                      b = df_mn.index[df_mn["motDesc"].isin(["TOTAL MOT",
                                                                'Other MOT'])]
                      t.loc[a, "netWgt"] = (mv[0] + nv[0])/2
                      t.drop(b,inplace = True)
                      cp_v7 = 1

            if len(m) > len(n) == 1: ## 2
                cp_v7 = 0
                for x in range(len(m)):
                    uxy = set([mm[x]]).union(set(nm))
                    if mm[x] == nm[0] or uxy == mot_s or uxy == mot_l:             ## have the same transport from the both sides
                       t.loc[m.index[x],
                                       "netWgt"] = (nv[0] + mv[x])/2
                       t.drop(n.index,inplace = True)
                       cp_v7 = 1
                if set(mm)&set(nm) == set() and cp_v7 == 0:                        ## No same transports from the two sides
                   av = (sum(mv) + sum(nv))/2
                   for x in range(len(m)):
                       t.loc[m.index[x], "netWgt"] = av*mv[x]/sum(mv)
                       cp_v7 = 1
                   t.drop(n.index,inplace = True)
                if cp_v7 == 0:
                    oths = pd.concat([m,n])
                    lst_om.append(oths)

            if len(n) > len(m) == 1: ## 3
                cp_v7 = 0
                for x in range(len(n)):
                    uxy = set([nm[x]]).union(set(mm))
                    if nm[x] == mm[0] or uxy == mot_s or uxy == mot_l:             ## have the same transport from the both sides
                       t.loc[n.index[x],
                                       "netWgt"] = (mv[0] + nv[x])/2
                       t.drop(m.index,inplace = True)
                       cp_v7 = 1
                if set(mm)&set(nm) == set() and cp_v7 == 0:                        ## No same transports from the two sides
                   av = (sum(mv) + sum(nv))/2
                   for x in range(len(n)):
                       t.loc[n.index[x], "netWgt"] = av*nv[x]/sum(nv)
                       cp_v7 = 1
                   t.drop(m.index,inplace = True)
                if cp_v7 == 0:
                    oths = pd.concat([n,m])
                    lst_om.append(oths)

            if len(m) > 1 and len(n) > 1: ## 4: len(p) or len(q) = 2 or 3
               cp_v7 = 0
               for i in itertools.product(m.index.tolist(), n.index.tolist()):
                   mi, ni = m.loc[i[0], "motDesc"], n.loc[i[1], "motDesc"]
                   miv, niv = m.loc[i[0],"netWgt"], n.loc[i[1],"netWgt"]
                   uxy = set([mi]).union(set([ni]))
                   if mi == ni or uxy == mot_s or uxy == mot_l:
                      t.loc[i[1],"netWgt"] = (miv + niv)/2
                      t.drop(i[0],inplace = True)
                      cp_v7 = 1

               if cp_v7 == 0:
                  av = (sum(mv) + sum(nv))/2
                  t.loc[n.index[x], "netWgt"] = av*nv[x]/sum(nv)
                  t.drop(m.index,inplace = True)
               if cp_v7 == 0:
                   oths = pd.concat([n,m])
                   lst_om.append(oths)

    df = t.drop_duplicates().reset_index(drop = True)
    lst_sum.append(df)

lst_sum = ['{:.3e}'.format(t["netWgt"].sum()) for t in lst_sum]
print("Average only: the netWeight of all plastic waste for 4 yrs are"
      f"\n {lst_sum} {time.strftime('%Y-%m-%d %H:%M:%S')}")                    ## Average only: the netWeight of all plastic waste for 4 yrs are ['1.474e+10', '1.655e+10', '1.542e+10', '1.627e+10', '1.374e+10', '9.091e+09', '8.428e+09', '7.687e+09', '7.803e+09', '7.534e+09']
                                                                                                                                                
end_time = time.time()
time_taken = '{:.2f}'.format((end_time - start_time)/60)                       
print(f"Time taken: {time_taken} mins {time.strftime('%Y-%m-%d %H:%M:%S')}")   ## Time taken: 386.74 mins

with open('./model/pkl/lst5_model.pkl', 'wb') as f:
    pickle.dump(lst5, f)                                                       ## picke the list lst5

with open('./model/pkl/lst5_model.pkl', 'rb') as f:
    lst5 = pickle.load(f)                                                      
#%% 9. Transfer X into M (exchange the position of reporter and partner)
lst6 = copy.deepcopy(lst5)   
for t in lst6:
    for i in t.index:
        if t.loc[i,"flowCode"] == "X":
            t.loc[i,"flowCode"] = "M"
            t.loc[i, "reporterDesc"], t.loc[i, "partnerDesc"] = \
            t.loc[i, "partnerDesc"], t.loc[i, "reporterDesc"] 
            t.loc[i, "reporterISO"], t.loc[i, "partnerISO"] = \
            t.loc[i, "partnerISO"], t.loc[i, "reporterISO"] 
            t.loc[i, "reporterCode"], t.loc[i, "partnerCode"] = \
            t.loc[i, "partnerCode"], t.loc[i, "reporterCode"]   
            t.loc[i, "flowDesc"] = 'Import'        
    t.drop((t[t['reporterDesc'] == t['partnerDesc']]).index,inplace = True)
    t.reset_index(drop = True,inplace = True)

#%% 10. Research country selection and trade share

lst7 = copy.deepcopy(lst6) 
lst_im =[]
lst_ex =[]
for i in lst7:
    lst_im.append(i.groupby(["reporterDesc"]).sum(numeric_only = True).loc[
        :,["netWgt","primaryValue"]].sort_values('netWgt',ascending= False)) 
    lst_ex.append(i.groupby(["partnerDesc"]).sum(numeric_only = True).loc[
        :,["netWgt","primaryValue"]].sort_values('netWgt',ascending= False)) 
              
def pc(d): 
    d.sort_values(["netWgt"], ascending = False, inplace = True)
    d["Percent"] = d["netWgt"]/d["netWgt"].sum()*100
    d["CPercent"] = d["Percent"].cumsum()
    d["Region"] = [get_key(i) for i in d.index]
    return d

lst_im =  [pc(i) for i in lst_im]
lst_ex =  [pc(i) for i in lst_ex]
         
## select countries taking 80% plastic waste exports during 2013-2022
df_aim = pd.concat(lst_im).groupby(['reporterDesc']).sum(numeric_only =True)
df_aim.sort_values(["netWgt"], ascending = False, inplace = True)
df_aim["Percent"] = df_aim["netWgt"]/df_aim["netWgt"].sum()*100
df_aim["CPercent"] = df_aim["Percent"].cumsum()
df_aim["Region"] = [get_key(i) for i in df_aim.index]
df_aim = df_aim[df_aim["CPercent"]<80]
df_aim['mode'] = 'M'

df_aex = pd.concat(lst_ex).groupby(['partnerDesc']).sum(numeric_only =True)
df_aex.sort_values(["netWgt"], ascending = False, inplace = True)
df_aex["Percent"] = df_aex["netWgt"]/df_aex["netWgt"].sum()*100
df_aex["CPercent"] = df_aex["Percent"].cumsum()
df_aex["Region"] = [get_key(i) for i in df_aex.index]
df_aex = df_aex[df_aex["CPercent"]<80]
df_aex['mode'] = 'X'

df_aim,df_aex = df_aim.add_prefix('m_'),df_aex.add_prefix('x_')
df_a = pd.concat([df_aim,df_aex],axis =1).reset_index()   ## combine countries taking 70% of imports or exports during 2013-2022
df_a['m_Region'] = df_a['m_Region'].combine_first(df_a['x_Region'])
df_a.drop(columns=['m_mode','x_mode','x_Region'],inplace = True)
len(df_a.index.unique()) 
##df_a.to_pickle("./model/pkl/df_a.pkl") 

## select countries taking 80% plastic waste exports during 2020-2022                                 ## this df_3 is selected
df_3im = pd.concat(lst_im[-3:]).groupby(['reporterDesc']).sum(numeric_only =True)
df_3im.sort_values(["netWgt"], ascending = False, inplace = True)
df_3im["Percent"] = df_3im["netWgt"]/df_3im["netWgt"].sum()*100
df_3im["CPercent"] = df_3im["Percent"].cumsum()
df_3im["Region"] = [get_key(i) for i in df_3im.index]
df_3im = df_3im[df_3im["CPercent"]<70]
df_3im['mode'] = 'M'

df_3ex = pd.concat(lst_ex[-3:]).groupby(['partnerDesc']).sum(numeric_only =True)
df_3ex.sort_values(["netWgt"], ascending = False, inplace = True)
df_3ex["Percent"] = df_3ex["netWgt"]/df_3ex["netWgt"].sum()*100
df_3ex["CPercent"] = df_3ex["Percent"].cumsum()
df_3ex["Region"] = [get_key(i) for i in df_3ex.index]
df_3ex = df_3ex[df_3ex["CPercent"]<70]
df_3ex['mode'] = 'X'

df_3im,df_3ex = df_3im.add_prefix('m_'),df_3ex.add_prefix('x_')
df_3 = pd.concat([df_3im,df_3ex],axis =1).reset_index()                        
df_3['m_Region'] = df_3['m_Region'].combine_first(df_3['x_Region'])
df_3.drop(columns=['m_mode','x_mode','x_Region'],inplace = True)
len(df_3.index.unique()) 
##df_3.to_pickle("./model/pkl/df_3.pkl") 

## select countries taking 80% plastic waste exports each year druing 2013-2022
lst_a =[]
for i,j in zip(lst_im, lst_ex):
    ## i,j = lst_im[0],lst_ex[0]
    a = i[i["CPercent"]<80].add_prefix('m_')
    b = j[j["CPercent"]<80].add_prefix('x_')
    c = pd.concat([a,b],axis =1).reset_index()  
    c['m_Region'] = c['m_Region'].combine_first(c['x_Region'])
    c.drop(columns=['x_Region'],inplace = True)
    lst_a.append(c)
    
with open('./model/pkl/lst_a.pkl', 'wb') as f:
    pickle.dump(lst_a, f)                                                      ## picke the list lst5
    
with open('./model/pkl/lst_a.pkl', 'rb') as f:
    lst_a = pickle.load(f)
    

## share of trade between selected countries in total trade during 2013-2022
df_s = pd.concat(lst7)
df_25 = df_s.loc[df_s['partnerDesc'].isin(df_a['index'])&df_s['reporterDesc'
                                                    ].isin(df_a['index'])]
t = '{:.2f}'.format(df_25.netWgt.sum()/df_s.netWgt.sum())
print('trade share of the selected 25 countries in total trade during 2013-2022'
      f' is {t} {time.strftime("%Y-%m-%d %H:%M:%S")}')                         ## trade share of the selected 25 countries in total trade during 2013-2022 
                                                                               ## is 0.75 2023-10-03 10:41:58

## share of trade between selected countries in total trade in each year
lst_s =[]
for i,j in zip(lst7,lst_a):
    df_25 = i.loc[i['partnerDesc'].isin(j['index'])&i['reporterDesc'
                                                        ].isin(j['index'])]
    t = '{:.2f}'.format(df_25.netWgt.sum()/i.netWgt.sum())
    lst_s.append(t)                                                            
print('Trade share between selected countries in each year'  ## Share of trade between selected countries in total trade in each year is 
      f' is {lst_s} {time.strftime("%Y-%m-%d %H:%M:%S")}')                     ## ['0.74', '0.74', '0.71', '0.74', '0.78', '0.74', '0.72', '0.71', '0.71', '0.69']
                                                                               ## 2023-10-03 10:40:30

## share of trade between selected countries in total trade during 2020-2022
df_s3 = pd.concat(lst7[-1:])
df = df_s3.loc[df_s3['partnerDesc'].isin(df_3['index'])&df_s3['reporterDesc'
                                                    ].isin(df_3['index'])]
t = '{:.2f}'.format(df.netWgt.sum()/df_s3.netWgt.sum())                                                         
print('Trade share between selected countries in'
      f' 2022 is {t} {time.strftime("%Y-%m-%d %H:%M:%S")}')                    ## Trade share between selected countries in 2022 is 0.56 2023-10-03 11:22:21

## share of trade exported from UK in 2022
df_s3 = pd.concat(lst7[-1:])
df = df_s3.loc[df_s3['partnerDesc']== 'United Kingdom']
df1 = df_s3.loc[(df_s3['partnerDesc']== 'United Kingdom')&~(df_s3['reporterDesc']).isin(dict7.get('EU', []))]
t = '{:.2f}'.format(df.netWgt.sum()/df_s3.netWgt.sum())  
t1 = '{:.2f}'.format(df1.netWgt.sum()/df.netWgt.sum())                                                     
print('Trade share of UK and its non-EU share in'
      f' 2022 is {t} and {t1} {time.strftime("%Y-%m-%d %H:%M:%S")}')           ## Trade share of UK and its non-EU share in 2022 is 0.08 and 0.38 2023-10-14 17:16:45


#%% 11. Build common transport modes for 26 research  countries
                                    
df = pd.read_pickle("./model/pkl/df10.pkl")                                

df_td = df[["reporterDesc", "partnerDesc", "motDesc"]].loc[
       (df["motDesc"] != "TOTAL MOT")&(df["partnerDesc"] != "World")&(
       ~df["motDesc"].isin(lst_omot))]

df_td = df_td.groupby(["reporterDesc", "partnerDesc", 
               "motDesc"]).size().reset_index().rename(columns ={0:"count"})
df_td.insert(1,"rtRg", [get_key(i) for i in df_td["reporterDesc"]])
df_td.insert(3,"ptRg", [get_key(i) for i in df_td["partnerDesc"]])

df_td.to_pickle("./model/pkl/df_td.pkl")
#%% 12. Use common transport modes to replace unspecified modes

lst8 =[]
for i in lst7:
    t = i[(i.reporterDesc.isin(list(df_3['index'])))&(
        i.partnerDesc.isin(list(df_3['index'])))].reset_index(drop = True)
    t = t[['period','reporterISO','reporterDesc','flowCode','partnerISO',
           'partnerDesc','cmdCode','motDesc','netWgt','primaryValue']]   
    lst8.append(t)
    
lst9= copy.deepcopy(lst8)                                                      ## For the deep copy, it means that any changes made to a copy of the object do not reflect in the original object. 

df_td = pd.read_pickle("./model/pkl/df_td.pkl")     
                       
v = df_td
m,n,j,k = v["reporterDesc"], v["partnerDesc"], v["rtRg"], v["ptRg"]
lst_v9r = []
lp = 0

for t in lst9:           
    for i in t.index:
        if t.loc[i,"motDesc"] in lst_omot:
            c1, c2 = t["reporterDesc"][i], t["partnerDesc"][i]                
            a = pd.concat([v[(m == c1)&(n == c2)], 
                             v[(m == c2)&(n == c1)]])                          ## search the common transport modes bwtween two countries
            
            b = pd.concat([v[(m == c1)&(k == get_key(c2))],                    ## search the common transport modes between one coutry and one region
                               v[(m == c2)&(k == get_key(c1))],
                               v[(j == get_key(c1))&(n == c2)],
                               v[(j == get_key(c2))&(n == c1)]])            

            if len(a) != 0:
                atm = a.loc[a["count"].idxmax()]["motDesc"]
                if isinstance(atm, pd.Series):                                 ## in cases several maximums exist, atm is a series format rather than string.
                    atm2 = b.loc[b["count"].idxmax()]["motDesc"]
                    if set(atm2).issubset(set(atm)) and len(atm2)==1:
                       atm = atm2
                    else:
                         atm = atm.iloc[0]
                else:
                     atm = atm
                t.loc[i,"motDesc"] = atm
                lp = lp + 1
                lst_v9r.append(t.loc[[i]])
            if len(a) == 0 and len(b) != 0:
                atm = b.loc[b["count"].idxmax()]["motDesc"]
                if isinstance(atm, pd.Series):
                    atm = atm.iloc[0]
                else:
                    atm = atm
                t.loc[i,"motDesc"] = atm
                lp = lp + 1
                lst_v9r.append(t.loc[[i]])
    
df_v9 = pd.concat(lst_v9r)

lst = [len(t[t["motDesc"].isin(lst_omot)]) for t in lst9]
print(f'Unspecified transport modes in lst9 are {lst},'
      f" {time.strftime('%Y-%m-%d %H:%M:%S')}")                                 ## Unspecified transport modes in lst9 are [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 2023-10-03 13:51:07

with open('./model/pkl/lst9.pkl', 'wb') as f:
    pickle.dump(lst9, f)                                                      ## picke the lst9
    
with open('./model/pkl/lst9.pkl', 'rb') as f:
    lst9 = pickle.load(f)
    
#%% 13. Life cycle inventory of transport

## Sea distance data (km) from the CERDI-seadistance database
## https://ferdi.fr/en/indicators/the-cerdi-seadistance-database

df_dbs = pd.read_excel("CERDI-seadistance.xlsx")

data = {'Name':['Germany', 'Canada', 'Spain','Malaysia', 'Türkiye', "Poland",
             'Other Asia, nes', 'Indonesia', 'Viet Nam','Netherlands','Mexico', 
      'United Kingdom', 'Belgium', 'France', 'Italy',  'USA', 'Japan','Austria'],
        
      'ISO3':["DEU", "CAN", "ESP",  "MYS", "TUR","POL",
              "TWN",  "IDN", "VNM", "NLD", "MEX", 
              "GBR", "BEL", "FRA", "ITA", "USA", "JPN","AUT"],      
      'ISO2':["DE", "CA", "ES",  "MY", "TR",  "PL",            
              "TW", "ID", "VN", "NL", "MX",
              "GB", "BE", "FR", "IT", "US", "JP","AT"]}

df_geo= pd.DataFrame(data)

## Sea distance

lst_ship = []
lst_ck= []
for i in itertools.combinations(df_geo["ISO3"], 2):
    data = df_dbs[(df_dbs["iso1"] == i[0])&(df_dbs["iso2"] == i[1])]
    if len(data) == 0:
        lst_ck.append(i)
    lst_ship.append(data)

df_ship = pd.concat(lst_ship)[["iso1", "iso2",
          "seadistance"]].rename(columns={"seadistance": 
          "Sea"}).reset_index(drop = True)                                      ## seadistance: bilateral sea distance in kilometers(numeric).

df_ship.insert(1, "c1", 0)
df_ship.insert(2, "iso1_b", [df_geo.loc[df_geo["ISO3"] == df_ship.loc[i,
                    "iso1"],"ISO2"].values[0] for i in df_ship.index])         ## add a column of ISO2 departure country
df_ship.insert(4, "c2", 0)     
df_ship.insert(5, "iso2_b", [df_geo.loc[df_geo["ISO3"] == df_ship.loc[i,       ## ## add a column of ISO2 destination country
                    "iso2"],"ISO2"].values[0] for i in df_ship.index])                                       
                                                                              


## Air distance (great-circle distance,km) calculated by spherical law of cosines

from math import radians, sin, cos, acos
def great_circle(lon1, lat1, lon2, lat2):
    lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])
    return 6371 * (acos(sin(lat1) * sin(lat2) + cos(lat1) * cos(lat2) * cos(lon1 - lon2)))

## road and railway distance (km) by Bing map API
url = 'https://raw.githubusercontent.com/google/dspl/master/samples/google/canonical/countries.csv'

df_co = pd.read_csv(url)


for i in df_ship.index:
    #i=6
    lat1 = df_co.loc[df_co["country"] == df_ship.loc[i,"iso1_b"], 
                     "latitude"].values[0]
    lon1 = df_co.loc[df_co["country"] == df_ship.loc[i,"iso1_b"], 
                     "longitude"].values[0]
    lat2 = df_co.loc[df_co["country"] == df_ship.loc[i,"iso2_b"], 
                     "latitude"].values[0]
    lon2 = df_co.loc[df_co["country"] == df_ship.loc[i,"iso2_b"], 
                     "longitude"].values[0]
    airdist = great_circle(lon1, lat1, lon2, lat2)
    
    url1 = "https://dev.virtualearth.net/REST/v1/Routes/DistanceMatrix?"\
        f"origins={lat1},{lon1}&destinations={lat2},{lon2}&travelMode=driving&"\
        "key=AqTVFLLIEkpBWruHpTXmeZyKPPBoDH6zajpI4bjmFZnA7wbVc-2HScYSXf7s6U-h"
        
    data_json = json.loads(urlopen(url1).read())   
    
    if data_json["resourceSets"][0]["resources"][0][
        "results"][0]["travelDuration"] > 0:
        df_ship.loc[i, "road"] = data_json["resourceSets"][0]["resources"][0][
            "results"][0]["travelDistance"]
    else:
         df_ship.loc[i, "road"] = float("nan")        
           
    df_ship.loc[i, "air"] = airdist

    df_ship.loc[i,"c1"] = df_geo.loc[df_geo["ISO3"] == df_ship["iso1"][i], 
                                     "Name"].values[0]                         ##  add a column of complete departure country names
    df_ship.loc[i,"c2"] = df_geo.loc[df_geo["ISO3"] == df_ship["iso2"][i], 
                                     "Name"].values[0]                         ## add a column of complete destination country names

df_ship.to_pickle("./model/pkl/df_ship.pkl")
df_ship = pd.read_pickle("./model/pkl/df_ship.pkl")  

"""
BAU: waste treated in the importing countries
EE: waste from European countries treated in European countries
EA: waste from European and North America countries treated demestically
OD: waste from OECD countries treated within OECD countries
TD: waste treated domestically 
"""
## Per unit environmental impact of transport
 
df_tra = pd.read_excel("./model/sce_trans.xlsx", header=None)                
 
lst = []
for i in df_ship.index:
    ## road
    if not pd.isna(df_ship.loc[i,"road"]):
     se_road = df_ship.loc[i,"road"]*df_tra.loc[0:1,7:].mean()
    if pd.isna(df_ship.loc[i,"road"]):
       se_road = pd.Series([np.nan] * 18, index=range(7, 25))
    ## train
    if not pd.isna(df_ship.loc[i,"road"]):
        se_train = df_ship.loc[i,"road"]*df_tra.loc[2:4, 7:].mean()
    if pd.isna(df_ship.loc[i,"road"]):
       se_train = pd.Series([np.nan] * 18, index=range(7, 25))
    ## sea
    if not pd.isna(df_ship.loc[i,"Sea"]):
       se_sea = df_ship.loc[i,"Sea"]*df_tra.loc[5:6, 7:].mean()
    if pd.isna(df_ship.loc[i,"Sea"]):
       se_sea = pd.Series([np.nan] * 18, index=range(7, 25))
    
    ## air
    if not pd.isna(df_ship.loc[i,"air"]):
       se_air = df_ship.loc[i,"air"]*df_tra.loc[7, 7:]
    if pd.isna(df_ship.loc[i,"air"]):
       se_air = pd.Series([np.nan] * 18, index=range(7, 25))
    ## concatenating
    a = pd.concat([se_sea, se_air, se_road, se_train])

    lst.append(a)

df = pd.concat(lst, axis = 1,).reset_index(drop = True).T

lst_EI = ['tap','gwp1000', 'fetp','metp','tetp','ffp','fep','mep','htpc',
          'htpnc','irp','lop','sop','odpinfinite','pmfp','hofp','eofp','wcp']

lst_name = ["sea_" + item for item in lst_EI] +["air_" + item for item in lst_EI]\
    + ["road_" + item for item in lst_EI] + ["train_" + item for item in lst_EI]

df.columns = lst_name

df_EItra = pd.concat([df_ship, df], axis = 1)

df_EItra.to_pickle("./model/pkl/df_EItra.pkl")
df_EItra = pd.read_pickle("./model/pkl/df_EItra.pkl")
#%% 14. Treatment environmental impacts
#%%% 14.1. Preparation

for j in lst9:
    for i in range(len(j)):
        ## j = lst9[0]
        ## i = 8
        if get_key7(j["reporterDesc"][i]) == 'EU' and get_key7(
                    j["partnerDesc"][i]) == 'EU':
            j.loc[i, "EU"] = 'intra-EU' 
        if get_key7(j["reporterDesc"][i]) != 'EU' and get_key7(
                    j["partnerDesc"][i]) == 'EU':
            j.loc[i, "EU"] = 'inter-EU'    
            
        if get_key7(j["reporterDesc"][i]) in ['EU','UK'] and get_key7(
                    j["partnerDesc"][i]) in ['EU','UK']:
            j.loc[i, "EU+UK"] = 'intra-EU (UK)'                 
        if get_key7(j["reporterDesc"][i]) not in ['EU','UK'] and get_key7(
                    j["partnerDesc"][i]) in ['EU','UK']:
            j.loc[i, "EU+UK"] = 'inter-EU (UK)' 
            
        if get_key8(j["reporterDesc"][i]) == 'OECD' and get_key8(
                    j["partnerDesc"][i]) == 'OECD':
            j.loc[i, "OECD"] = 'intra-OECD' 
        if get_key8(j["reporterDesc"][i]) != 'OECD' and get_key8(
                    j["partnerDesc"][i]) == 'OECD':
            j.loc[i, "OECD"] = 'inter-OECD' 
            
        if get_key7(j["partnerDesc"][i]) == 'EU' and get_key8(
           j["reporterDesc"][i]) != 'OECD' :
            j.loc[i, "EU-OECD"] = 'EU-to-non-OECD'         
        if get_key7(j["partnerDesc"][i]) == 'EU' and get_key8(
           j["reporterDesc"][i]) == 'OECD' :
            j.loc[i, "EU-OECD"] = 'EU-to-OECD' 
            
        
        if get_key7(j["partnerDesc"][i]) in ['EU','UK'] and get_key8(
           j["reporterDesc"][i]) != 'OECD' :
            j.loc[i, "EU(UK)-OECD"] = 'EU(UK)-to-non-OECD'         
        if get_key7(j["partnerDesc"][i]) in ['EU','UK'] and get_key8(
           j["reporterDesc"][i]) == 'OECD' :
            j.loc[i, "EU(UK)-OECD"] = 'EU(UK)-to-OECD' 
            
            
        if get_key9(j["reporterDesc"][i]) == 'mismanage':
            j.loc[i, "mismanage"] = 'to mismanage'  
        if get_key9(j["reporterDesc"][i]) != 'mismanage': 
            j.loc[i, "mismanage"] = 'non-mismanage' 
        

            
        if j.loc[i,"cmdCode"] == "391510":
           j.loc[i,"cmdCode"] = "PE"
        if j.loc[i,"cmdCode"] == "391520":
           j.loc[i,"cmdCode"] = "PS"
        if j.loc[i,"cmdCode"] == "391530":
           j.loc[i,"cmdCode"] = "PVC"
        if j.loc[i,"cmdCode"] == "391590":
           j.loc[i,"cmdCode"] = "Others"


## Read the environmenatl impact intensity table exported from Activity-Browser
df_lca = pd.read_excel("./model/sce_rt4.xlsx", index_col = 0, header = [0,1]) 

lst=[]                                                                
for i in df_lca.index:
    lst.append(i.split('|')[0].strip())  
df_lca.index = lst

df_lca = df_lca.stack(level=[0, 1]).reset_index()                              ## turn two levels of column headers into two columns
df_lca = df_lca.pivot(index = ['method', 'scenario'], columns = ['level_0'])   ## put one column into one column header
df_lca.columns = df_lca.columns.droplevel(0)
df_lca = df_lca.reset_index().rename(columns ={'scenario':'country'})
df_lca = df_lca.drop(df_lca[df_lca.country == 'Default'].index)

lst=[]                                                                
for i in df_lca.method:
    ## i = df_lca.method[0]
    lst.append(i.split(',')[-1].strip().capitalize())  
df_lca.method = lst


## Read required recycling rate
df_lp= pd.read_pickle("./BC/pkl/df_lp.pkl")

df_lp = df_lp[['reporterDesc','period','cmdCode','rate']]
df_lp1 = df_lp.pivot(index = ['reporterDesc','period'], columns = 'cmdCode', 
                     values = 'rate').reset_index()
df_lp2 = df_lp1.groupby(['reporterDesc']).median(numeric_only= True).reset_index()
##df_lp2.to_excel('./model/df_lp2.xlsx')

## Read treatment share
df_sn = pd.read_excel("./model/scenario_table2.xlsx")  
df_sr = pd.read_excel("./model/scenario_table_source.xlsx", sheet_name = 'num')  
df_sr.drop(columns = 'Year',inplace = True)


lst_a = df_lca.method.unique().tolist()
lst_b = [i.split('(')[-1].split(')')[0] for i in lst_a]
lst_d = ['tre_'+i for i in lst_b]
def ei(c,cmd,rate):
    ## c = 'Poland'
    ## rate = '100%'
    if c == 'Austria' or 'Japan':
        r_ot, r_pe, r_ps, r_pvc = df_lp2.loc[df_lp2.reporterDesc.isin([
     'Belgium','France','Germany','Italy','Netherlands','Spain','United Kingdom']),
     ['Others','PE','PS','PVC']].mean(numeric_only= True)
    if c == 'Poland':
        r_ot, r_pe, r_ps, r_pvc = df_lp2.loc[df_lp2.reporterDesc.isin([
     'Slovenia','Czechia']),['Others','PE','PS','PVC']].mean(numeric_only= True)

    
    if c not in [ 'Austria' , 'Japan' , 'Poland']:
     r_ot, r_pe, r_ps, r_pvc = df_lp2.loc[df_lp2.reporterDesc == c,\
                                         ['Others','PE','PS','PVC']].values[0]
        
    if c == 'Türkiye':
       c = 'Turkey'      
       
    s_r, s_iwe, s_iwoe, s_ls, s_lu, s_od, s_ob = df_sr.loc[
        df_sr.Country == c].mean(numeric_only=True).values/100
    
    if rate =='av':
       r_ot = r_pe = r_ps = r_pvc = s_r
        
    if rate =='100%':
       r_ot = r_pe = r_ps = r_pvc = 1

    r_HDPE,r_LDPE,r_PS,r_PVC,r_PET,r_PP = 0.82, 0.81, 0.82, 0.82,0.82,0.83

    ## s_r, s_iwe, s_iwoe, s_ls, s_lu, s_od, s_ob = 
    ## df_sr.loc[df_sr.loc[df_sr['Country'] == c,'s_r'].idxmax()]              ## retrieve the highest s_r
    
    if cmd == 'PE':        
       HDPE, LDPE =df_sn.loc[df_sn.Name.isin(['HDPE','LDPE']),c]
       i_HDg1, i_HDg2, i_HDrgL, i_HDrgU, i_LDg1, i_LDg2, i_LDrgL, i_LDrgU,\
       i_iwoL, i_iwL, i_iwU, i_iwoU, i_ob,i_od,i_rrL,i_rrU, i_sl, i_ul =\
       df_lca.loc[df_lca.country == c, 
                  [col for col in df_lca.columns if "PE " in col]].T.values         
       ei_rH = r_pe*HDPE*(i_HDrgL+i_HDrgU)/2-r_pe*HDPE*r_HDPE*(i_HDg1+i_HDg2)/2
       ei_rL = r_pe*LDPE*(i_LDrgL+i_LDrgU)/2-r_pe*LDPE*r_LDPE*(i_LDg1+i_LDg2)/2                                                                                      
       ei_iwe = (1-r_pe)*s_iwe*(1/(1-s_r))*(i_iwL+i_iwU)/2*(-1)
       ei_iwoe = (1-r_pe)*s_iwoe*(1/(1-s_r))*(i_iwoL+i_iwoU)/2*(-1)
       ei_sl = (1-r_pe)*s_ls*(1/(1-s_r))*i_sl*(-1)
       ei_ul = (1-r_pe)*s_lu*(1/(1-s_r))*i_ul*(-1)
       ei_od = (1-r_pe)*s_od*(1/(1-s_r))*i_od*(-1)
       ei_ob = (1-r_pe)*s_ob*(1/(1-s_r))*i_ob*(-1)
       ei = ei_rH + ei_rL + ei_iwe + ei_iwoe + ei_sl + ei_ul + ei_od + ei_ob
       lst_c = ['_rH', '_rL', '_iwe' ,'_iwoe', '_sl', '_ul', '_od', '_ob']
       for i in range(18):
           n = [j[i] for j in [ei_rH, ei_rL, ei_iwe, ei_iwoe ,ei_sl , ei_ul , ei_od , ei_ob]]
           ei = np.append(ei, n)
       names = lst_d + [s1 + s2 for s1 in lst_b for s2 in lst_c]
       series = pd.Series(ei, index=names)
       
    if cmd == 'PS':
       i_g1,i_g2,i_iwL,i_iwoL,i_iwU,i_iwoU,i_ob,i_od,i_rrL,i_rrU,i_rgL, i_rgU,\
           i_sl,i_ul =df_lca.loc[df_lca.country == c, 
                  [col for col in df_lca.columns if "PS " in col]].T.values         
       ei_r = r_ps*(i_rgL+i_rgU)/2-r_ps*r_PS*(i_g1+i_g2)/2
       ei_iwe = (1-r_ps)*s_iwe*(1/(1-s_r))*(i_iwL+i_iwU)/2*(-1)
       ei_iwoe = (1-r_ps)*s_iwoe*(1/(1-s_r))*(i_iwoL+i_iwoU)/2*(-1)
       ei_sl = (1-r_ps)*s_ls*(1/(1-s_r))*i_sl*(-1)
       ei_ul = (1-r_ps)*s_lu*(1/(1-s_r))*i_ul*(-1)
       ei_od = (1-r_ps)*s_od*(1/(1-s_r))*i_od*(-1)
       ei_ob = (1-r_ps)*s_ob*(1/(1-s_r))*i_ob*(-1)
       ei = ei_r + ei_iwe + ei_iwoe + ei_sl + ei_ul + ei_od + ei_ob      
       lst_c = ['_r', '_iwe' , '_iwoe' ,'_sl' , '_ul' , '_od' , '_ob']
      
       for i in range(18):
           n = [j[i] for j in [ei_r, ei_iwe , ei_iwoe ,ei_sl , ei_ul , ei_od , ei_ob]]
           ei = np.append(ei, n)
       names = lst_d + [s1 + s2 for s1 in lst_b for s2 in lst_c]
       series = pd.Series(ei, index=names)

    if cmd == 'PVC':
       i_g1,i_g2, i_iwL,i_iwoL,i_iwU,i_iwoU,i_ob,i_od,i_rrL,i_rrU,i_rgL, i_rgU,\
          i_sl,i_ul =df_lca.loc[df_lca.country == c, 
                  [col for col in df_lca.columns if "PVC " in col]].T.values         
       ei_r = r_pvc*(i_rgL+i_rgU)/2-r_pvc*r_PVC*(i_g1+i_g2)/2                                                                                        
       ei_iwe = (1-r_pvc)*s_iwe*(1/(1-s_r))*(i_iwL+i_iwU)/2*(-1)
       ei_iwoe = (1-r_pvc)*s_iwoe*(1/(1-s_r))*(i_iwoL+i_iwoU)/2*(-1)
       ei_sl = (1-r_pvc)*s_ls*(1/(1-s_r))*i_sl*(-1)
       ei_ul = (1-r_pvc)*s_lu*(1/(1-s_r))*i_ul*(-1)
       ei_od = (1-r_pvc)*s_od*(1/(1-s_r))*i_od*(-1)
       ei_ob = (1-r_pvc)*s_ob*(1/(1-s_r))*i_ob*(-1)
       ei = ei_r + ei_iwe + ei_iwoe + ei_sl + ei_ul + ei_od + ei_ob
       lst_c = ['_r', '_iwe' , '_iwoe' ,'_sl' , '_ul' , '_od' , '_ob']
       for i in range(18):
           n = [j[i] for j in [ei_r, ei_iwe, ei_iwoe ,ei_sl , ei_ul , ei_od , ei_ob]]
           ei = np.append(ei, n)
       names = lst_d + [s1 + s2 for s1 in lst_b for s2 in lst_c]
       series = pd.Series(ei, index=names)
       
    if cmd == 'Others':
       PET, PP = df_sn.loc[df_sn.Name.isin(['PET','PP']),c]
       i_Tg1, i_Tg2, i_TiwL,i_TiwoL,i_TiwU,i_TiwoU, i_Tob,i_Tod,i_TrrL,i_TrrU,\
       i_TrgL, i_TrgU, i_Tsl, i_Tul, i_Pg1, i_Pg2, i_PiwL,i_PiwoL,i_PiwU,\
       i_PiwoU, i_Pob,i_Pod,i_PrrL,i_PrrU,i_PrgL, i_PrgU, i_Psl, i_Pul,=\
       df_lca.loc[df_lca.country == c, 
     [col for col in df_lca.columns if "PET " in col or "PP " in col]].T.values      
       
       ei_rT = r_ot*PET*(i_TrgL+i_TrgU)/2-r_ot*PET*r_PET*(i_Tg1+i_Tg2)/2
       ei_rP = r_ot*PP*(i_PrgL+i_PrgU)/2-r_ot*PP*r_PP*(i_Pg1+i_Pg2)/2  
                                                                                    
       ei_iweT = PET*(1-r_ot)*s_iwe*(1/(1-s_r))*(i_TiwL+i_TiwU)/2*(-1)     
       ei_iweP = PP*(1-r_ot)*s_iwe*(1/(1-s_r))*(i_PiwL+i_PiwU)/2*(-1)
       
       ei_iwoeT = PET*(1-r_ot)*s_iwoe*(1/(1-s_r))*(i_TiwoL+i_TiwoU)/2*(-1)
       ei_iwoeP = PP*(1-r_ot)*s_iwoe*(1/(1-s_r))*(i_PiwoL+i_PiwoU)/2*(-1)
       
       ei_slT = (1-r_ot)*s_ls*(1/(1-s_r))*i_Tsl*(-1)
       ei_slP = (1-r_ot)*s_ls*(1/(1-s_r))*i_Psl*(-1)
       
       ei_ulT = (1-r_ot)*s_lu*(1/(1-s_r))*i_Tul*(-1)
       ei_ulP = (1-r_ot)*s_lu*(1/(1-s_r))*i_Pul*(-1)
       
       ei_odT = (1-r_ot)*s_od*(1/(1-s_r))*i_Tod*(-1)     
       ei_odP = (1-r_ot)*s_od*(1/(1-s_r))*i_Pod*(-1)
       
       ei_obT = (1-r_ot)*s_ob*(1/(1-s_r))*i_Tob*(-1)
       ei_obP = (1-r_ot)*s_ob*(1/(1-s_r))*i_Pob*(-1)
       
       ei = ei_rT + ei_rP + ei_iweT + ei_iweP + ei_iwoeT + ei_iwoeP + ei_slT +\
            ei_slP + ei_ulT + ei_ulP + ei_odT + ei_odP + ei_obT + ei_obP
       lst_c = ['_rT', '_rP', '_iweT', '_iweP', '_iwoeT', '_iwoeP', '_slT', '_slP', \
                           '_ulT', '_ulP', '_odT', '_odP', '_obT', '_obP']
       for i in range(18):
           n  = [j[i] for j in [ei_rT, ei_rP, ei_iweT, ei_iweP, ei_iwoeT, ei_iwoeP, ei_slT, ei_slP, \
                               ei_ulT, ei_ulP, ei_odT, ei_odP, ei_obT, ei_obP]]
           ei = np.append(ei, n)
       names = lst_d + [s1 + s2 for s1 in lst_b for s2 in lst_c]
       series = pd.Series(ei, index=names)
            
    return series

## link impact of transport
def ei_tra(t):
    for i in t.index:
        ## i = t.index[0]    
        a = df_EItra.loc[(df_EItra["c1"] == t["reporterDesc"][i])&
                       (df_EItra["c2"] == t["partnerDesc"][i])]     
        if len(a) == 0:
            a = df_EItra.loc[(df_EItra["c2"] == t["reporterDesc"][i])&
                           (df_EItra["c1"] == t["partnerDesc"][i])]

        if t["motDesc"][i] in ["Sea", "Water"] and \
            t["reporterDesc"][i] != t["partnerDesc"][i]:
            lst_c = [col for col in a.columns if "sea_" in col]
            t.loc[i, c] = [t.netWgt[i]*a[x].values[0]/1000 for x in lst_c]
        
        elif t["motDesc"][i] in ["Road", "Land"] and \
            t["reporterDesc"][i] != t["partnerDesc"][i]:
            lst_c = [col for col in a.columns if "road_" in col]
            t.loc[i, c] = [t.netWgt[i]*a[x].values[0]/1000 for x in lst_c]
            if t.loc[i, c].isnull().all():                                         ## if the specified road or land transport is not geographically feasible, instead use sea transport
                lst_c = [col for col in a.columns if "sea_" in col]
                t.loc[i, c] = [t.netWgt[i]*a[x].values[0]/1000 for x in lst_c]           

        elif t["motDesc"][i] == "Railway" and \
            t["reporterDesc"][i] != t["partnerDesc"][i]:
            lst_c = [col for col in a.columns if "train_" in col]
            t.loc[i, c] = [t.netWgt[i]*a[x].values[0]/1000 for x in lst_c]
            if t.loc[i, c].isnull().all():                                  ## if the specified railway transport is not geographically feasible, instead use road transport
                lst_c = [col for col in a.columns if "road_" in col]
                t.loc[i, c] = [t.netWgt[i]*a[x].values[0]/1000 for x in lst_c]
                if t.loc[i, c].isnull().all():                              ## if the road transport is also not geographically feasible, instead use sea transport. 
                    lst_c = [col for col in a.columns if "sea_" in col]
                    t.loc[i, c] = [t.netWgt[i]*a[x].values[0]/1000 
                                          for x in lst_c]                                            

        elif t["motDesc"][i] == "Air" and \
            t["reporterDesc"][i] != t["partnerDesc"][i]:
            lst_c = [col for col in a.columns if "air_" in col]
            t.loc[i, c] = [t.netWgt[i]*a[x].values[0]/1000 for x in lst_c]
            
        else:
            t.loc[i, c] = pd.DataFrame(18*[np.nan])   
    return t


## Function of spliting netweights into single rows by treatment for sankey diagram
## test = t.groupby(['reporterDesc', 'partnerDesc','cmdCode'])[['netWgt']].sum(numeric_only= True).reset_index()
def nw(row,rate):
    ##row =test.loc[0,:]
    ## rate =0
    if rate == '100%':       
       r, iwe, iwoe, ls, lu, od, ob = 1, 0,0,0,0,0,0
    if rate == 'avg':
       c = row['partnerDesc']
       if c == 'Türkiye':
          c = 'Turkey' 
       r, iwe, iwoe, ls, lu, od,ob = df_sr.loc[df_sr.Country == c].mean(numeric_only=True).values/100
    if rate in ['100%','avg']:
        my_dict = {'r': r,'iwe':iwe, 'iwoe':iwoe, 'ls':ls, 'lu':lu, 'od':od, 'ob':ob}
        rows = []
        for t, s in my_dict.items():
         new_row = row.copy()
         new_row['sweight'] = row['netWgt'] * s
         new_row['treatment'] = t
         rows.append(new_row)
    if rate == 0:   
       c = row['reporterDesc']
    if rate == 'td3':
       c = row['partnerDesc']
    if rate in [0,'td3']:
       if c == 'Austria' or 'Japan':
           r_ot, r_pe, r_ps, r_pvc = df_lp2.loc[df_lp2.reporterDesc.isin([
        'Belgium','France','Germany','Italy','Netherlands','Spain','United Kingdom']),
        ['Others','PE','PS','PVC']].mean(numeric_only= True)
       if c == 'Poland':
           r_ot, r_pe, r_ps, r_pvc = df_lp2.loc[df_lp2.reporterDesc.isin([
        'Slovenia','Czechia']),['Others','PE','PS','PVC']].mean(numeric_only= True)

       
       if c not in [ 'Austria' , 'Japan' , 'Poland']:
        r_ot, r_pe, r_ps, r_pvc = df_lp2.loc[df_lp2.reporterDesc == c,\
                                            ['Others','PE','PS','PVC']].values[0]
           
       if c == 'Türkiye':
          c = 'Turkey'      
          
       s_r, s_iwe, s_iwoe, s_ls, s_lu, s_od, s_ob = df_sr.loc[
           df_sr.Country == c].mean(numeric_only=True).values/100
       
       for i,j in zip(['PE','PS','PVC','Others'],[r_pe, r_ps,r_pvc,r_ot]):
           if row['cmdCode'] == i:
              r, iwe, iwoe, ls, lu, od, ob = j, (1-j)*s_iwe*(1/(1-s_r)), (1-j)*s_iwoe*(1/(1-s_r)), \
              (1-j)*s_ls*(1/(1-s_r)),(1-j)*s_lu*(1/(1-s_r)),(1-j)*s_od*(1/(1-s_r)),(1-j)*s_ob*(1/(1-s_r))
              my_dict = {'r': r,'iwe':iwe, 'iwoe':iwoe, 'ls':ls, 'lu':lu, 'od':od, 'ob':ob}
              rows = []
              for t, s in my_dict.items():
               new_row = row.copy()
               new_row['sweight'] = row['netWgt'] * s
               new_row['treatment'] = t
               rows.append(new_row)             
    
    return rows                         
#%%% 14.2 BAU: waste treated in the importing countries

lst10= copy.deepcopy(lst9) 
a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]
c = ["tra_" + item for item in b]
t = lst10[-1]
##for i in b:
   ## t[i] = np.nan

           
lst = []
for i in t.index:
    ## i = t.index[0]
    a1 = ei(t.reporterDesc[i],t.cmdCode[i],0)*t.netWgt[i]
    lst.append(a1)
    
df = pd.concat(lst, axis=1, keys=[f'Series{i+1}' for i, s in enumerate(lst)])
df = df.T.reset_index(drop = True)

dfs= copy.deepcopy(df)  
for i in ['_r','_iwe','_iwoe','_sl','_ul','_od','_ob']:
    ##  i = '_od'
    for j in lst_b:
        ## j = 'odpinfinite'
        lst = [col for col in dfs.columns if i in col and j in col 
               and 'tre_' not in col]
        k = j+'__'+i
        dfs[k] = dfs[lst].sum(axis=1)
        dfs.drop(columns = lst,inplace = True)

t = ei_tra(t)
df_bau = pd.concat([t, dfs], axis=1)

for i in df_bau.index:
    df_bau = df_bau.drop(df_bau[df_bau['reporterDesc'] == df_bau['partnerDesc']].index)                 ## drop the trade with the same reporter and partner
    
t1 = df_bau.groupby(['reporterDesc']).sum(numeric_only = True)

t1.loc['total_bau',:] = t1.sum()

t1s = copy.deepcopy(t1)                                                        ## merge treatment and transport impacts
for i in b:                                                        
    ## i = b[0]
    lst = [col for col in t1s.columns if i in col if 'tra' in col or 'tre' in col]
    t1s[i] = t1s[lst].sum(axis=1)
    t1s.drop(columns = lst[1],inplace = True)
order = ['netWgt','primaryValue']+[col for prefix in lst_b for col in t1s.columns if prefix in col]
df_1_bau = t1s[order]    
    
#%%% 14.3 TD1: waste treated domestically with demestic treatment mix
lst10= copy.deepcopy(lst9) 
t = lst10[-1]

lst = []
for i in t.index:
    ## i = t.index[0]
    a1 = ei(t.partnerDesc[i],t.cmdCode[i], 'av')*t.netWgt[i]
    lst.append(a1)

df = pd.concat(lst, axis=1, keys=[f'Series{i+1}' for i, s in enumerate(lst)])
df = df.T.reset_index(drop = True)

dfs= copy.deepcopy(df)  
for i in ['_r','_iwe','_iwoe','_sl','_ul','_od','_ob']:
    ##  i = '_r'
    for j in lst_b:
        ## j = 'irp'
        lst = [col for col in dfs.columns if i in col and j in col 
               and 'tre_' not in col]
        k = j+'__'+i
        dfs[k] = dfs[lst].sum(axis=1)
        dfs.drop(columns = lst,inplace = True)

df_td1 = pd.concat([t, dfs], axis=1)
        
for i in df_td1.index:
    df_td1 = df_td1.drop(df_td1[df_td1['reporterDesc'] == df_td1['partnerDesc']].index)                 ## drop the trade with the same reporter and partner
        
        
t2 = df_td1.groupby(['partnerDesc']).sum(numeric_only = True)

t2.loc['total_td1',:] = t2.sum()

t2s = copy.deepcopy(t2)                                                        ## merge treatment and transport impacts
for i in b:                                                        
    ## i = b[0]
    lst = [col for col in t2s.columns if i in col if 'tra' in col or 'tre' in col]
    t2s[i] = t2s[lst].sum(axis=1)
    t2s.drop(columns = lst,inplace = True)
order = ['netWgt','primaryValue']+[col for prefix in lst_b for col in t2s.columns if prefix in col]
df_2_td = t2s[order]    

#%%% 14.4 TD2: waste treated domestically with 100% recycling rate
lst10= copy.deepcopy(lst9) 
t = lst10[-1]

lst = []
for i in t.index:
    ## i = t.index[0]
    a1 = ei(t.partnerDesc[i],t.cmdCode[i], '100%')*t.netWgt[i]
    lst.append(a1)

df = pd.concat(lst, axis=1, keys=[f'Series{i+1}' for i, s in enumerate(lst)])
df = df.T.reset_index(drop = True)

dfs= copy.deepcopy(df)  
for i in ['_r','_iwe','_iwoe','_sl','_ul','_od','_ob']:
    ##  i = 'od'
    for j in lst_b:
        ## j = 'odpinfinite'
        lst = [col for col in dfs.columns if i in col and j in col 
               and 'tre_' not in col]
        k = j+'__'+i
        dfs[k] = dfs[lst].sum(axis=1)
        dfs.drop(columns = lst,inplace = True)

df_td2 = pd.concat([t, dfs], axis=1)
        
for i in df_td2.index:
    df_td2 = df_td2.drop(df_td2[df_td2['reporterDesc'] == df_td2['partnerDesc']].index)                 ## drop the trade with the same reporter and partner
        
        
t3 = df_td2.groupby(['partnerDesc']).sum(numeric_only = True)

t3.loc['total_td2',:] = t3.sum()

t3s = copy.deepcopy(t3)                                                        ## merge treatment and transport impacts
for i in b:                                                        
    ## i = b[0]
    lst = [col for col in t3s.columns if i in col if 'tra' in col or 'tre' in col]
    t3s[i] = t3s[lst].sum(axis=1)
    t3s.drop(columns = lst,inplace = True)
order = ['netWgt','primaryValue']+[col for prefix in lst_b for col in t3s.columns if prefix in col]
df_3_td = t3s[order]    

#%%% 14.5 TD3: waste treated domestically with imported plastic waste recycling rate
lst10= copy.deepcopy(lst9) 
t = lst10[-1]

lst = []
for i in t.index:
    ## i = t.index[0]
    a1 = ei(t.partnerDesc[i],t.cmdCode[i], 0)*t.netWgt[i]
    lst.append(a1)

df = pd.concat(lst, axis=1, keys=[f'Series{i+1}' for i, s in enumerate(lst)])
df = df.T.reset_index(drop = True)

dfs= copy.deepcopy(df)  
for i in ['_r','_iwe','_iwoe','_sl','_ul','_od','_ob']:
    ##  i = '_r'
    for j in lst_b:
        ## j = 'irp'
        lst = [col for col in dfs.columns if i in col and j in col 
               and 'tre_' not in col]
        k = j+'__'+i
        dfs[k] = dfs[lst].sum(axis=1)
        dfs.drop(columns = lst,inplace = True)

df_td3 = pd.concat([t, dfs], axis=1)
        
for i in df_td3.index:
    df_td3 = df_td3.drop(df_td3[df_td3['reporterDesc'] == df_td3['partnerDesc']].index)                 ## drop the trade with the same reporter and partner
        
        
t4 = df_td3.groupby(['partnerDesc']).sum(numeric_only = True)

t4.loc['total_td3',:] = t4.sum()

t4s = copy.deepcopy(t4)                                                        ## merge treatment and transport impacts
for i in b:                                                        
    ## i = b[0]
    lst = [col for col in t4s.columns if i in col if 'tra' in col or 'tre' in col]
    t4s[i] = t4s[lst].sum(axis=1)
    t4s.drop(columns = lst,inplace = True)
order = ['netWgt','primaryValue']+[col for prefix in lst_b for col in t4s.columns if prefix in col]
df_4_td = t4s[order]  

#%%% 14.6 Comparison 
lst =[]
for i in [df_1_bau, df_2_td,df_3_td,df_4_td]:
    a = i.iloc[-1]
    lst.append(a)
df = pd.concat(lst,axis = 1)

## difference of other scenarios to BAU
##for i in df.columns[1:]:
    ##df[i] =  df[i] - df.iloc[:,0]

df1 = df.loc[[col for col in df.index if '__' not in col if 'tra' not in col],:]

df2 = copy.deepcopy(df1)

for n,i in df2.iterrows():
    for j in range(1,len(i)):
        if i[0]<0:
            i[j] =  -(i[j]-i[0])/i[0]*100
        else:i[j] =  (i[j]-i[0])/i[0]*100
        
##df2.loc[:,'total_bau'] = 1
df2[df2.columns[0]] = 1
df3 = pd.concat([df2,df1],axis =1)

df3.to_excel('./model/df3_4.xlsx')

#%% 15. Plots
#%%% 15.1 Sankey diagram a.BAU (country-country-treatment)
"""
Asian country group:
Other Asia exporter: #FFAB91 | Shadow Color: #FFD3C0
Turkey: #FF7043 | Shadow Color: #FF9470
Malaysia: #FFA726 | Shadow Color: #FFC085
Vietnam: #FFCA28 | Shadow Color: #FFE09F
Indonesia: #FFEE58 | Shadow Color: #FFF7AE
Other Asia importer: #FFD600 | Shadow Color: #FFE544

European country group:
Germany: #81D4FA | Shadow Color: #B3E5FC
Netherlands: #4FC3F7 | Shadow Color: #A2D5F2
United Kingdom: #29B6F6 | Shadow Color: #92C9E8
Other Europe exporter: #039BE5 | Shadow Color: #66B2D3
Other Europe importer: #0288D1 | Shadow Color: #579AC8

Japan: #969696 | Shadow Color: #d9d9d9

USA:: #7E57C2 | Shadow Color: #B39DDB

Mechanical Recycling: #76A79F (Desaturated Green)
Incineration: #c1747b
Sanitary Landfill: #e2c395
Unsanitary Landfill: #b6865e
Open Dump: #7f5b3f
Incineration (without energy for recovery): #995c65
Open Burning: #833839
process_colors = ['#76A79F', '#c1747b', '#e2c395', '#b6865e','#7f5b3f', 
                  '#D995c65','#833839']

plastic type: plastic_colors = ['#9FB3E8', '#EEDC9F', '#93cf9f', '#D1A7A1']
shadow_colors = ['#BFC9F2', '#F3E7B2', '#add9b1', '#E5C3BB']

"""
lst10= copy.deepcopy(lst9) 
a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]
c = ["tra_" + item for item in b]
t = lst10[-1]

t = t.groupby(['reporterDesc', 'partnerDesc','cmdCode'])[['netWgt']].sum(numeric_only= True
                                                                         ).reset_index()
lst = [nw(row,0) for _, row in t.iterrows()] 

sk = pd.DataFrame([item for sublist in lst for item in sublist]).reset_index(drop = True)
sk = sk.drop(sk[sk.sweight == 0].index).reset_index(drop = True)
sk = sk.groupby(['reporterDesc', 'partnerDesc','treatment'])['sweight'].sum(numeric_only= True
                                                                            ).reset_index()

lst = ['North America','Germany','Netherlands','United Kingdom','Other Europe exporter',
       'Japan','Other Asia exporter']
lst1 = ['North America','Netherlands','Germany','Other Europe importer','Türkiye',
        'Malaysia','Viet Nam', 'Indonesia','Other Asia importer']

se_1 = pd.Series(lst, name = "rtRg").reset_index(drop = True)
se_2 = pd.Series(lst1, name = "rtRg").reset_index(drop = True)
se_2.index += 7
se_3 = pd.Series(['r','iwe','ls','lu','od','iwoe','ob'], name = "treat")
se_3.index += 16

lst_cl2 = ["#B39DDB", "#B3E5FC","#A2D5F2","#92C9E8",'#66B2D3','#d9d9d9',
        '#FFD3C0','#579AC8','#FF9470','#FFC085','#FFE09F','#FFF7AE','#FFE544']
lst_cl = ["#7E57C2", "#81D4FA","#4FC3F7","#29B6F6",'#039BE5','#969696',
        '#FFAB91','#0288D1','#FF7043','#FFA726','#FFCA28','#FFEE58','#FFD600']

lst_n3 =['#76A79F', '#D1AF77', '#AFAFAF', '#BF8A73', '#918482', '#D1A477', '#AA7474']

data1 = {"color_node": lst_cl,
        "color_shadow": lst_cl2,
        'region': lst+[i for i in lst1 if i not in lst]}

df_cl = pd.DataFrame(data1)

t_1= sk.groupby(["partnerDesc","reporterDesc"])["sweight"].sum(numeric_only = True).reset_index()
t_1.insert(1,'region',[get_key10(i) for i in t_1.partnerDesc])
t_1.insert(3,'region_y',[get_key11(i) for i in t_1.reporterDesc])
t_1=t_1.groupby(["region","region_y"])["sweight"].sum(numeric_only = True
                                ).reset_index().reset_index(drop = True)


t_2= sk.groupby(["reporterDesc","treatment"])["sweight"].sum(numeric_only = True).reset_index()
t_2.insert(1,'region_y',[get_key11(i) for i in t_2.reporterDesc])
t_2=t_2.groupby(["region_y","treatment"])["sweight"].sum(numeric_only = True
                                ).reset_index().reset_index(drop = True)


lst_so, lst_ta, lst_v,lst_clk= [],[],[],[]
for i in t_1.index:        
    so = se_1[se_1 == t_1.loc[i, "region"]].index[0]
    ta = se_2[se_2 == t_1.loc[i, "region_y"]].index[0]      
    v = t_1.loc[i, "sweight"]
    clk = df_cl.loc[df_cl["region"] == t_1.loc[i, "region"],
           "color_shadow"].values[0]
    lst_so.append(so)        
    lst_ta.append(ta)      
    lst_v.append(v)        
    lst_clk.append(clk) 
    
    
lst_so2, lst_ta2, lst_v2,lst_clk2= [],[],[],[]
for i in t_2.index:        
    so = se_2[se_2 == t_2.loc[i, "region_y"]].index[0]
    ta = se_3[se_3 == t_2.loc[i, "treatment"]].index[0]      
    v = t_2.loc[i, "sweight"]
    clk = df_cl.loc[df_cl["region"] == t_2.loc[i, "region_y"],
           "color_shadow"].values[0]   
    lst_so2.append(so)        
    lst_ta2.append(ta)      
    lst_v2.append(v)        
    lst_clk2.append(clk) 
    
f = np.linspace(0.01, 0.99, 7).tolist()
r = np.linspace(0.01, 0.99, 9).tolist()
p = np.linspace(0.01, 0.99, 7).tolist()

mynode = dict(
      pad = 15,  thickness = 15, color = ["#7E57C2", "#81D4FA","#4FC3F7",
    "#29B6F6",'#039BE5','#969696','#FFAB91',"#7E57C2","#4FC3F7","#81D4FA",
    '#0288D1','#FF7043','#FFA726','#FFCA28','#FFEE58','#FFD600']+lst_n3, 
      line = dict(color = "black", width = 0.5),
     label = se_1.tolist()+se_2.tolist()+se_3.tolist(), 
      x = [0.01]*7 +[0.3]*9+[0.6]*7, y = f+r+p)         ## to narrow the width (x) and save the space for extra annotations

fig = go.Figure(data=[go.Sankey(
  arrangement = "snap",
  node = mynode,
  link = dict(source = lst_so+lst_so2, target = lst_ta+lst_ta2,                            ## indices correspond to labels, eg A1, A2, A1, B1, ...
              value = lst_v+lst_v2, color = lst_clk+lst_clk2) )])

lst_s = ['{:.0f}'.format(t_2[t_2["treatment"]==f"{i}"].sum(
       numeric_only = True).values[0]/t["netWgt"].sum(
       numeric_only = True)*100) for i in se_3.tolist()]

fig.update_layout(title_text = 
    f"a.BAU (country-country-treatment) in 2022"
    f"(r: {lst_s[0]}%, iwe: {lst_s[1]}%, ls: {lst_s[2]}%,"
    f" lu: {lst_s[3]}%, od: {lst_s[4]}%, iwoe: {lst_s[5]}%, ob: {lst_s[6]}%)", 
    font_size=10)
    
fig.show()
fig.write_image('./model/svg/sk_a_bau.svg')
#%%% 15.2 Sankey diagram b.BAU (country-type-country)

lst10= copy.deepcopy(lst9) 
a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]
c = ["tra_" + item for item in b]
t = lst10[-1]

t = t.groupby(['reporterDesc', 'partnerDesc','cmdCode'])[['netWgt']].sum(numeric_only= True
                                                                         ).reset_index()


lst = ['North America','Germany','Netherlands','United Kingdom','Other Europe exporter',
       'Japan','Other Asia exporter']
lst1 = ['North America','Netherlands','Germany','Other Europe importer','Türkiye',
        'Malaysia','Viet Nam', 'Indonesia','Other Asia importer']

se_1 = pd.Series(lst, name = "rtRg").reset_index(drop = True)
se_2 = pd.Series(['PE','PS','PVC','Others'], name = "type").reset_index(drop = True)
se_2.index += 7
se_3 = pd.Series(lst1, name = "rtRg").reset_index(drop = True)
se_3.index += 11

lst_cl2 = ["#B39DDB", "#B3E5FC","#A2D5F2","#92C9E8",'#66B2D3','#d9d9d9',
        '#FFD3C0','#BFC9F2', '#F3E7B2', '#A9CBD3', '#E5C3BB']
lst_cl = ["#7E57C2", "#81D4FA","#4FC3F7","#29B6F6",'#039BE5','#969696',
        '#FFAB91','#9FB3E8', '#EEDC9F', '#8AB6C1', '#D1A7A1']


lst_n3 =["#7E57C2", "#4FC3F7", '#81D4FA', '#0288D1', '#FF7043','#FFA726',
         '#FFCA28','#FFEE58','#FFD600']

data1 = {"color_node": lst_cl,
        "color_shadow": lst_cl2,
        'region': lst+list(se_2)}

df_cl = pd.DataFrame(data1)


t.insert(1,'region',[get_key10(i) for i in t.partnerDesc])
t.insert(3,'region_y',[get_key11(i) for i in t.reporterDesc])
t_1=t.groupby(["region","cmdCode"])["netWgt"].sum(numeric_only = True
                                ).reset_index().reset_index(drop = True)

t_2= t.groupby(["cmdCode","region_y"])["netWgt"].sum(
    numeric_only = True).reset_index()


lst_so, lst_ta, lst_v,lst_clk= [],[],[],[]
for i in t_1.index:        
    so = se_1[se_1 == t_1.loc[i, "region"]].index[0]
    ta = se_2[se_2 == t_1.loc[i, "cmdCode"]].index[0]      
    v = t_1.loc[i, "netWgt"]
    clk = df_cl.loc[df_cl["region"] == t_1.loc[i, "region"],
           "color_shadow"].values[0]
    lst_so.append(so)        
    lst_ta.append(ta)      
    lst_v.append(v)        
    lst_clk.append(clk) 
    
    
lst_so2, lst_ta2, lst_v2,lst_clk2= [],[],[],[]
for i in t_2.index:        
    so = se_2[se_2 == t_2.loc[i, "cmdCode"]].index[0]
    ta = se_3[se_3 == t_2.loc[i, "region_y"]].index[0]      
    v = t_2.loc[i, "netWgt"]
    clk = df_cl.loc[df_cl["region"] == t_2.loc[i, "cmdCode"],
           "color_shadow"].values[0]   
    lst_so2.append(so)        
    lst_ta2.append(ta)      
    lst_v2.append(v)        
    lst_clk2.append(clk) 
    
f = np.linspace(0.01, 0.99, 7).tolist()
r = [0.13, 0.42, 0.56, 0.84]
p = np.linspace(0.01, 0.99, 9).tolist()



mynode = dict(
      pad = 15,  thickness = 15, color = lst_cl+lst_n3, 
      line = dict(color = "black", width = 0.5),
     label = se_1.tolist()+se_2.tolist()+se_3.tolist(), 
      x = [0.01]*7 +[0.3]*4+[0.6]*9, y = f+r+p)         ## to narrow the width (x) and save the space for extra annotations

fig = go.Figure(data=[go.Sankey(
  arrangement = "snap",
  node = mynode,
  link = dict(source = lst_so+lst_so2, target = lst_ta+lst_ta2,                            ## indices correspond to labels, eg A1, A2, A1, B1, ...
              value = lst_v+lst_v2, color = lst_clk+lst_clk2) )])

lst_s = ['{:.0f}'.format(t_1[t_1["cmdCode"]==f"{i}"].sum(
       numeric_only = True).values[0]/t["netWgt"].sum(
       numeric_only = True)*100) for i in ["PE", "PS", "PVC", "Others"]]

fig.update_layout(title_text = 
    f"b.BAU (country-type-country) in 2022"
    f"(PE: {lst_s[0]}%, PS: {lst_s[1]}%, PVC: {lst_s[2]}%,"
    f"and Others: {lst_s[3]}%)", font_size=10)
    
fig.show()
fig.write_image('./model/svg/sk_b_bau.svg')
#%%% 15.3 Sankey diagram c.TD1 (treated domestically with 100% mechanical recycling) 
"""
Asian country group:
Other Asia exporter: #FFAB91 | Shadow Color: #FFD3C0
Turkey: #FF7043 | Shadow Color: #FF9470
Malaysia: #FFA726 | Shadow Color: #FFC085
Vietnam: #FFCA28 | Shadow Color: #FFE09F
Indonesia: #FFEE58 | Shadow Color: #FFF7AE
Other Asia importer: #FFD600 | Shadow Color: #FFE544

European country group:
Germany: #81D4FA | Shadow Color: #B3E5FC
Netherlands: #4FC3F7 | Shadow Color: #A2D5F2
United Kingdom: #29B6F6 | Shadow Color: #92C9E8
Other Europe exporter: #039BE5 | Shadow Color: #66B2D3
Other Europe importer: #0288D1 | Shadow Color: #579AC8

Japan: #969696 | Shadow Color: #d9d9d9

USA:: #7E57C2 | Shadow Color: #B39DDB

Mechanical Recycling: #76A79F (Desaturated Green)
Incineration: #c1747b
Sanitary Landfill: #e2c395
Unsanitary Landfill: #b6865e
Open Dump: #7f5b3f
Incineration (without energy for recovery): #995c65
Open Burning: #833839
process_colors = ['#76A79F', '#c1747b', '#e2c395', '#b6865e','#7f5b3f', 
                  '#D995c65','#833839']

plastic type: plastic_colors = ['#9FB3E8', '#EEDC9F', '#93cf9f', '#D1A7A1']
shadow_colors = ['#BFC9F2', '#F3E7B2', '#add9b1', '#E5C3BB']

"""
lst10= copy.deepcopy(lst9) 
a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]
c = ["tra_" + item for item in b]
t = lst10[-1]

t = t.groupby(['reporterDesc', 'partnerDesc','cmdCode'])[['netWgt']].sum(numeric_only= True
                                                                         ).reset_index()
lst = [nw(row,'100%') for _, row in t.iterrows()] 

sk = pd.DataFrame([item for sublist in lst for item in sublist]).reset_index(drop = True)
sk = sk.drop(sk[sk.sweight == 0].index).reset_index(drop = True)
sk = sk.groupby(['reporterDesc', 'partnerDesc','treatment'])['sweight'].sum(numeric_only= True
                                                                            ).reset_index()

lst = ['North America','Germany','Netherlands','United Kingdom','Other Europe exporter',
       'Japan','Other Asia exporter']

se_1 = pd.Series(lst, name = "rtRg").reset_index(drop = True)
se_2 = pd.Series(['r','iwe','ls','lu','od','iwoe','ob'], name = "type").reset_index(drop = True)
se_2.index += 7

lst_cl2 = ["#B39DDB", "#B3E5FC","#A2D5F2","#92C9E8",'#66B2D3','#d9d9d9',
        '#FFD3C0']
lst_cl = ["#7E57C2", "#81D4FA","#4FC3F7","#29B6F6",'#039BE5','#969696',
        '#FFAB91']

lst_n3 =['#76A79F', '#D1AF77', '#AFAFAF', '#BF8A73', '#918482', 
                  '#D1A477', '#AA7474']

data1 = {"color_node": lst_cl,
        "color_shadow": lst_cl2,
        'region': lst}

df_cl = pd.DataFrame(data1)


sk.insert(1,'region',[get_key10(i) for i in sk.partnerDesc])
t_1=sk.groupby(["region","treatment"])["sweight"].sum(numeric_only = True
                                ).reset_index().reset_index(drop = True)

lst_so, lst_ta, lst_v,lst_clk= [],[],[],[]
for i in t_1.index:        
    so = se_1[se_1 == t_1.loc[i, "region"]].index[0]
    ta = se_2[se_2 == t_1.loc[i, "treatment"]].index[0]      
    v = t_1.loc[i, "sweight"]
    clk = df_cl.loc[df_cl["region"] == t_1.loc[i, "region"],
           "color_shadow"].values[0]
    lst_so.append(so)        
    lst_ta.append(ta)      
    lst_v.append(v)        
    lst_clk.append(clk) 
        
    

f = np.linspace(0.01, 0.99, 7).tolist()

p = np.linspace(0.01, 0.99, 7).tolist()


mynode = dict(
      pad = 15,  thickness = 15, color = lst_cl+lst_n3, 
      line = dict(color = "black", width = 0.5),
     label = se_1.tolist()+se_2.tolist()+se_3.tolist(), 
      x = [0.01]*7 +[0.25]*7, y = f+p)         ## to narrow the width (x) and save the space for extra annotations

fig = go.Figure(data=[go.Sankey(
  arrangement = "snap",
  node = mynode,
  link = dict(source = lst_so, target = lst_ta,                            ## indices correspond to labels, eg A1, A2, A1, B1, ...
              value = lst_v, color = lst_clk) )])

lst_s = ['{:.0f}'.format(t_1[t_1["treatment"]==f"{i}"].sum(
       numeric_only = True).values[0]/t["netWgt"].sum(
       numeric_only = True)*100) for i in se_2.tolist()]

fig.update_layout(title_text = 
    f"c.TD1 (treated domestically with 100% mechanical recycling) in 2022"
    f"(r: {lst_s[0]}%, iwe: {lst_s[1]}%, ls: {lst_s[2]}%,"
    f" lu: {lst_s[3]}%, od: {lst_s[4]}%, iwoe: {lst_s[5]}%, ob: {lst_s[6]}%)", 
    font_size=10)    
    
fig.show()
fig.write_image('./model/svg/sk_c_td1.svg')
#%%% 15.4 Sankey diagram d.TD2 (treated domestically with recycling rate of imported waste)
"""
Asian country group:
Other Asia exporter: #FFAB91 | Shadow Color: #FFD3C0
Turkey: #FF7043 | Shadow Color: #FF9470
Malaysia: #FFA726 | Shadow Color: #FFC085
Vietnam: #FFCA28 | Shadow Color: #FFE09F
Indonesia: #FFEE58 | Shadow Color: #FFF7AE
Other Asia importer: #FFD600 | Shadow Color: #FFE544

European country group:
Germany: #81D4FA | Shadow Color: #B3E5FC
Netherlands: #4FC3F7 | Shadow Color: #A2D5F2
United Kingdom: #29B6F6 | Shadow Color: #92C9E8
Other Europe exporter: #039BE5 | Shadow Color: #66B2D3
Other Europe importer: #0288D1 | Shadow Color: #579AC8

Japan: #969696 | Shadow Color: #d9d9d9

USA:: #7E57C2 | Shadow Color: #B39DDB

Mechanical Recycling: #76A79F (Desaturated Green)
Incineration: #c1747b
Sanitary Landfill: #e2c395
Unsanitary Landfill: #b6865e
Open Dump: #7f5b3f
Incineration (without energy for recovery): #995c65
Open Burning: #833839
process_colors = ['#76A79F', '#c1747b', '#e2c395', '#b6865e','#7f5b3f', 
                  '#D995c65','#833839']

plastic type: plastic_colors = ['#9FB3E8', '#EEDC9F', '#93cf9f', '#D1A7A1']
shadow_colors = ['#BFC9F2', '#F3E7B2', '#add9b1', '#E5C3BB']

"""
lst10= copy.deepcopy(lst9) 
a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]
c = ["tra_" + item for item in b]
t = lst10[-1]

t = t.groupby(['reporterDesc', 'partnerDesc','cmdCode'])[['netWgt']].sum(numeric_only= True
                                                                         ).reset_index()
lst = [nw(row,'td3') for _, row in t.iterrows()] 

sk = pd.DataFrame([item for sublist in lst for item in sublist]).reset_index(drop = True)
sk = sk.drop(sk[sk.sweight == 0].index).reset_index(drop = True)
sk = sk.groupby(['reporterDesc', 'partnerDesc','treatment'])['sweight'].sum(numeric_only= True
                                                                            ).reset_index()

lst = ['North America','Germany','Netherlands','United Kingdom','Other Europe exporter',
       'Japan','Other Asia exporter']

se_1 = pd.Series(lst, name = "rtRg").reset_index(drop = True)
se_2 = pd.Series(['r','iwe','ls','lu','od','iwoe','ob'], name = "type").reset_index(drop = True)
se_2.index += 7

lst_cl2 = ["#B39DDB", "#B3E5FC","#A2D5F2","#92C9E8",'#66B2D3','#d9d9d9',
        '#FFD3C0']
lst_cl = ["#7E57C2", "#81D4FA","#4FC3F7","#29B6F6",'#039BE5','#969696',
        '#FFAB91']

lst_n3 =['#76A79F', '#D1AF77', '#AFAFAF', '#BF8A73', '#918482', 
                  '#D1A477', '#AA7474']

data1 = {"color_node": lst_cl,
        "color_shadow": lst_cl2,
        'region': lst}

df_cl = pd.DataFrame(data1)


sk.insert(1,'region',[get_key10(i) for i in sk.partnerDesc])
t_1=sk.groupby(["region","treatment"])["sweight"].sum(numeric_only = True
                                ).reset_index().reset_index(drop = True)

lst_so, lst_ta, lst_v,lst_clk= [],[],[],[]
for i in t_1.index:        
    so = se_1[se_1 == t_1.loc[i, "region"]].index[0]
    ta = se_2[se_2 == t_1.loc[i, "treatment"]].index[0]      
    v = t_1.loc[i, "sweight"]
    clk = df_cl.loc[df_cl["region"] == t_1.loc[i, "region"],
           "color_shadow"].values[0]
    lst_so.append(so)        
    lst_ta.append(ta)      
    lst_v.append(v)        
    lst_clk.append(clk) 
        
    

f = np.linspace(0.01, 0.99, 7).tolist()

p = np.linspace(0.01, 0.99, 7).tolist()


mynode = dict(
      pad = 15,  thickness = 15, color = lst_cl+lst_n3, 
      line = dict(color = "black", width = 0.5),
     label = se_1.tolist()+se_2.tolist()+se_3.tolist(), 
      x = [0.01]*7 +[0.25]*7, y = f+p)         ## to narrow the width (x) and save the space for extra annotations

fig = go.Figure(data=[go.Sankey(
  arrangement = "snap",
  node = mynode,
  link = dict(source = lst_so, target = lst_ta,                            ## indices correspond to labels, eg A1, A2, A1, B1, ...
              value = lst_v, color = lst_clk) )])

lst_s = ['{:.0f}'.format(t_1[t_1["treatment"]==f"{i}"].sum(
       numeric_only = True).values[0]/t["netWgt"].sum(
       numeric_only = True)*100) for i in se_2.tolist()]

fig.update_layout(title_text = 
    f"d.TD2 (treated domestically with recycling rate of imported waste) in 2022"
    f"(r: {lst_s[0]}%, iwe: {lst_s[1]}%, ls: {lst_s[2]}%,"
    f" lu: {lst_s[3]}%, od: {lst_s[4]}%, iwoe: {lst_s[5]}%, ob: {lst_s[6]}%)", 
    font_size=10)  
fig.show()
fig.write_image('./model/svg/sk_d_td2.svg')
#%%% 15.5 Sankey diagram e.TD3 (treated domestically with national mix treatments)
"""
Asian country group:
Other Asia exporter: #FFAB91 | Shadow Color: #FFD3C0
Turkey: #FF7043 | Shadow Color: #FF9470
Malaysia: #FFA726 | Shadow Color: #FFC085
Vietnam: #FFCA28 | Shadow Color: #FFE09F
Indonesia: #FFEE58 | Shadow Color: #FFF7AE
Other Asia importer: #FFD600 | Shadow Color: #FFE544

European country group:
Germany: #81D4FA | Shadow Color: #B3E5FC
Netherlands: #4FC3F7 | Shadow Color: #A2D5F2
United Kingdom: #29B6F6 | Shadow Color: #92C9E8
Other Europe exporter: #039BE5 | Shadow Color: #66B2D3
Other Europe importer: #0288D1 | Shadow Color: #579AC8

Japan: #969696 | Shadow Color: #d9d9d9

USA:: #7E57C2 | Shadow Color: #B39DDB

Mechanical Recycling: #76A79F (Desaturated Green)
Incineration: #c1747b
Sanitary Landfill: #e2c395
Unsanitary Landfill: #b6865e
Open Dump: #7f5b3f
Incineration (without energy for recovery): #995c65
Open Burning: #833839
process_colors = ['#76A79F', '#c1747b', '#e2c395', '#b6865e','#7f5b3f', 
                  '#D995c65','#833839']

plastic type: plastic_colors = ['#9FB3E8', '#EEDC9F', '#93cf9f', '#D1A7A1']
shadow_colors = ['#BFC9F2', '#F3E7B2', '#add9b1', '#E5C3BB']

"""
lst10= copy.deepcopy(lst9) 
a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]
c = ["tra_" + item for item in b]
t = lst10[-1]

t = t.groupby(['reporterDesc', 'partnerDesc','cmdCode'])[['netWgt']].sum(numeric_only= True
                                                                         ).reset_index()
lst = [nw(row,'avg') for _, row in t.iterrows()] 

sk = pd.DataFrame([item for sublist in lst for item in sublist]).reset_index(drop = True)
sk = sk.drop(sk[sk.sweight == 0].index).reset_index(drop = True)
sk = sk.groupby(['reporterDesc', 'partnerDesc','treatment'])['sweight'].sum(numeric_only= True
                                                                            ).reset_index()

lst = ['North America','Germany','Netherlands','United Kingdom','Other Europe exporter',
       'Japan','Other Asia exporter']

se_1 = pd.Series(lst, name = "rtRg").reset_index(drop = True)
se_2 = pd.Series(['r','iwe','ls','lu','od','iwoe','ob'], name = "type").reset_index(drop = True)
se_2.index += 7

lst_cl2 = ["#B39DDB", "#B3E5FC","#A2D5F2","#92C9E8",'#66B2D3','#d9d9d9',
        '#FFD3C0']
lst_cl = ["#7E57C2", "#81D4FA","#4FC3F7","#29B6F6",'#039BE5','#969696',
        '#FFAB91']

lst_n3 =['#76A79F', '#D1AF77', '#AFAFAF', '#BF8A73', '#918482', 
                  '#D1A477', '#AA7474']

data1 = {"color_node": lst_cl,
        "color_shadow": lst_cl2,
        'region': lst}

df_cl = pd.DataFrame(data1)


sk.insert(1,'region',[get_key10(i) for i in sk.partnerDesc])
t_1=sk.groupby(["region","treatment"])["sweight"].sum(numeric_only = True
                                ).reset_index().reset_index(drop = True)

lst_so, lst_ta, lst_v,lst_clk= [],[],[],[]
for i in t_1.index:        
    so = se_1[se_1 == t_1.loc[i, "region"]].index[0]
    ta = se_2[se_2 == t_1.loc[i, "treatment"]].index[0]      
    v = t_1.loc[i, "sweight"]
    clk = df_cl.loc[df_cl["region"] == t_1.loc[i, "region"],
           "color_shadow"].values[0]
    lst_so.append(so)        
    lst_ta.append(ta)      
    lst_v.append(v)        
    lst_clk.append(clk) 
        

f = np.linspace(0.01, 0.99, 7).tolist()

p = np.linspace(0.01, 0.99, 7).tolist()


mynode = dict(
      pad = 15,  thickness = 15, color = lst_cl+lst_n3, 
      line = dict(color = "black", width = 0.5),
     label = se_1.tolist()+se_2.tolist()+se_3.tolist(), 
      x = [0.01]*7 +[0.25]*7, y = f+p)         ## to narrow the width (x) and save the space for extra annotations

fig = go.Figure(data=[go.Sankey(
  arrangement = "snap",
  node = mynode,
  link = dict(source = lst_so, target = lst_ta,                            ## indices correspond to labels, eg A1, A2, A1, B1, ...
              value = lst_v, color = lst_clk) )])

lst_s = ['{:.0f}'.format(t_1[t_1["treatment"]==f"{i}"].sum(
       numeric_only = True).values[0]/t["netWgt"].sum(
       numeric_only = True)*100) for i in se_2.tolist()]

fig.update_layout(title_text = 
    f"e.TD3 (treated domestically with national mix treatments) in 2022"
    f"(r: {lst_s[0]}%, iwe: {lst_s[1]}%, ls: {lst_s[2]}%,"
    f" lu: {lst_s[3]}%, od: {lst_s[4]}%, iwoe: {lst_s[5]}%, ob: {lst_s[6]}%)", 
    font_size=10)
    
fig.show()
#%%% 15.6 Horizontal bar chart (4)
plt.rcParams['pdf.fonttype'] = 42
plt.rcParams['ps.fonttype'] = 42

lst =[]
for i in [df_3_td,df_4_td,df_2_td,df_1_bau]:
    a = i.iloc[-1,2:]
    lst.append(a)
df = pd.concat(lst,axis = 1)

a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]

colors = ['#a1b3dc','#77a79f', '#c1747b', '#995c65', '#e2c395', '#b6865e', 
          '#7f5b3f', '#833839']                                                ##  Define a list of 8 colors

fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(5, 8))     # Set up the subplots grid
##fig, axes = plt.subplots(nrows=6, ncols=3, figsize=(9, 20))     # Set up the subplots grid

b= ['gwp1000','fetp','metp','tetp']
for i, category in enumerate(b):
    ## i,category = 0,'tap'
    ax = axes[i // 2][i % 2]
    ## ax = axes[i // 3][i % 3]
    relevant_rows = sorted([idx for idx in df.index if category in idx 
                and '_' in idx], key=lambda x: '0' if 'tra_' in x else '1')    ## Extract data corresponding to the current category. Transport to be the first. 

    sub_df = df.loc[relevant_rows].transpose()                                 # Transpose to make scenarios the index

    positive_df = sub_df.clip(lower=0)                                         ##  # Split positive and negative
    negative_df = sub_df.clip(upper=0)
    
    positive_df.plot(kind='bar', stacked=True, ax=ax, color=colors, 
                     legend=False,width = 0.5)                                 ## Plot
    negative_df.plot(kind='bar', stacked=True, ax=ax, color=colors, 
                     legend=False,width = 0.5)
    
    max_abs_value = max(abs(ax.get_ylim()[0]), abs(ax.get_ylim()[1]))          # Find the max absolute value
    ax.set_ylim(-max_abs_value, max_abs_value)                                 # Ensure the x-axis is symmetric around zero
    ax.axhline(0, color='#484949', linestyle='-',linewidth = 0.5) 

    net_emissions = sub_df.sum(axis=1)                                         # Compute net emissions
    
    ax.scatter(range(len(net_emissions)),net_emissions, color='#484949', 
               s=45, zorder=5)                                                 ## Plot a solid point at the net emission for each scenario
   
    side = np.sign(net_emissions).value_counts().idxmax()          # Determine the side for annotations based on the sign of most net emissions
    

    ylim = ax.get_ylim()                                                       # Get the x-limit to align annotations
    align_y = ylim[1] - 0.05 * (ylim[1] - ylim[0]
            ) if side < 0 else ylim[0] + 0.05 * (ylim[1] - ylim[0])

    
    for j, scenario in enumerate(net_emissions.index):
        rel_value = net_emissions[scenario] / net_emissions['total_bau']
        annotation = f"x{rel_value:.1f}" if rel_value>0 else\
            f"x({rel_value:.1f})"
        ax.text(j, align_y, annotation, va='center', 
                ha='right' if side < 0 else 'left')
    
    ax.set_title(category)
    # Use scientific notation for x-axis labels
    ax.ticklabel_format(axis='y', style='scientific', scilimits=(0,0))
    ax.set_xlabel('')
    custom_labels = ["TD1","TD2","TD3","BAU"]
    ax.set_xticklabels(custom_labels, rotation=0)
    

handles, _ = ax.get_legend_handles_labels()
labels = ['Transport','r','iwe','iwoe','sl','ul', 'od', 'ob']
fig.legend(handles[:8], labels[:8], loc='upper center', 
           bbox_to_anchor=(0.5, 0), ncol=5)
    
# Leave space at the top for the legend

# Adjust the layout
plt.tight_layout()
fig.savefig("./model/svg/hb_1.pdf", format='pdf')
##fig.savefig("./model/svg/hb.pdf", format='pdf')

#%%% 15.7 Horizontal bar chart (14)
plt.rcParams['pdf.fonttype'] = 42
plt.rcParams['ps.fonttype'] = 42

lst =[]
for i in [df_3_td,df_4_td,df_2_td,df_1_bau]:
    a = i.iloc[-1,2:]
    lst.append(a)
df = pd.concat(lst,axis = 1)

a = df_lca.method.unique().tolist()
b = [i.split('(')[-1].split(')')[0] for i in a]

colors = ['#a1b3dc','#77a79f', '#c1747b', '#995c65', '#e2c395', '#b6865e', 
          '#7f5b3f', '#833839']                                                ##  Define a list of 8 colors


c = ['gwp1000','fetp','metp','tetp']
d = [i for i in b if i not in c]

fig, axes = plt.subplots(nrows=3, ncols=5, figsize=(8, 12))     # Set up the subplots grid

for i, category in enumerate(d):
    ## i,category = 0,'tap'
    ax = axes[i // 5][i % 5]

    relevant_rows = sorted([idx for idx in df.index if category in idx 
                and '_' in idx], key=lambda x: '0' if 'tra_' in x else '1')    ## Extract data corresponding to the current category. Transport to be the first. 

    sub_df = df.loc[relevant_rows].transpose()                                 # Transpose to make scenarios the index

    positive_df = sub_df.clip(lower=0)                                         ##  # Split positive and negative
    negative_df = sub_df.clip(upper=0)
    
    positive_df.plot(kind='bar', stacked=True, ax=ax, color=colors, 
                     legend=False,width = 0.5)                                 ## Plot
    negative_df.plot(kind='bar', stacked=True, ax=ax, color=colors, 
                     legend=False,width = 0.5)
    
    max_abs_value = max(abs(ax.get_ylim()[0]), abs(ax.get_ylim()[1]))          # Find the max absolute value
    ax.set_ylim(-max_abs_value, max_abs_value)                                 # Ensure the x-axis is symmetric around zero
    ax.axhline(0, color='#484949', linestyle='-',linewidth = 0.5) 

    net_emissions = sub_df.sum(axis=1)                                         # Compute net emissions
    
    ax.scatter(range(len(net_emissions)),net_emissions, color='#484949', 
               s=30, zorder=5)                                                 ## Plot a solid point at the net emission for each scenario

    
    ax.set_title(category)
    # Use scientific notation for x-axis labels
    ax.ticklabel_format(axis='y', style='scientific', scilimits=(0,0))
    ax.set_xlabel('')
    custom_labels = ["TD1","TD2","TD3","BAU"]
    ax.set_xticklabels(custom_labels, rotation=0)
    

handles, _ = ax.get_legend_handles_labels()
labels = ['Transport','r','iwe','iwoe','sl','ul', 'od', 'ob']
fig.legend(handles[:8], labels[:8], loc='upper center', 
           bbox_to_anchor=(0.5, 0), ncol=5)
    
# Leave space at the top for the legend

# Adjust the layout
plt.tight_layout()
fig.savefig("./model/svg/hb14.pdf", format='pdf')
#%%% 15.8 Sankey diagrams of impact
## data preparation 
i = df_1_bau
a = i[[col for col in i.columns if 'gwp1000' in col 
or 'fetp' in col or 'metp' in col or 'tetp' in col]].iloc[:-1]
all_results = []
for column in a.columns:
    ## column = a.columns[1]
    # Sort the data by the indicator and separate top 3 and the rest
    sorted_df = a[column].abs().sort_values(ascending=False).index
    top_values = a.loc[sorted_df[:3], [column]]
    others_value = a.loc[sorted_df[3:], column].sum()
    if '___r' in column:
        top_values = a.loc[sorted_df[:3], [column]] 
        others_value = a.loc[sorted_df[3:], column].sum()
        
    # Aggregate values for 'Others'
    others_df = pd.DataFrame({column: [others_value]}, index=['Others'])

    # Concatenate top 3 and Others and append to all_results
    combined = pd.concat([top_values, others_df])
    all_results.append(combined)

result_df = pd.concat(all_results, axis=1)

df_melted = result_df.reset_index().rename(columns={"index": "Country"}
                    ).melt(id_vars=['Country'], value_vars=a.columns)
df_melted = df_melted.dropna()
df_melted['Total'] = df_melted.groupby('variable')['value'].transform('sum')      

## Plotting 

plt.rcParams['pdf.fonttype'] = 42
plt.rcParams['ps.fonttype'] = 42

lst = ['gwp1000','fetp','metp','tetp']
lst1 =[]

for i in lst:
    a = [f"{i}___ob", f"{i}___iwe", f"tra_{i}", f"{i}___r"]
    lst1.append(a)

lst_i = lst1[2]                                                                ##  change elements of list1 to compute between ['gwp1000','fetp','metp','tetp'
t = df_melted[df_melted['variable'].isin(lst_i)].reset_index(drop=True)     

# Convert the 'variable' column to a categorical type with specified ordering
t['variable'] = pd.Categorical(t['variable'], categories=lst_i, ordered=True)    ## used to apply the absolute value sorting only to the specified column. 
# Sort by the 'variable' column
##t = t.sort_values(['variable','value']).reset_index(drop=True)

t = t.sort_values(by=["variable", 'value'], 
         key=lambda x: x.abs() if x.name == 'value' else x)

# Function to apply the suffix
def apply_suffix(country_series):
    # Store seen countries and their counts
    seen = {}
    result = []
    
    for country in country_series:
        if country in seen:
            seen[country] += 1
            result.append(country + "_" + chr(96 + seen[country]))             # chr(96 + count) converts 1 to 'a', 2 to 'b', etc.
        else:
            seen[country] = 1
            result.append(country)
    
    return result

# Apply the function
t['Country'] = apply_suffix(t['Country'])

lst_se1 = t.Country.tolist()
         
se_1 = pd.Series(lst_i, name = "ei").reset_index(drop = True)
se_2 = pd.Series(lst_se1, name = "c").reset_index(drop = True)
se_2.index += 4

lst_cl2 = ["#ce9396", "#f2d6d9","#cedaf2","#a8ccc5"]
lst_cl = ["#833839", "#c1747b","#a1b3dc","#77a79f",]

data1 = {"color_node": lst_cl,
        "color_shadow": lst_cl2,
        'node': lst_i}
df_cl = pd.DataFrame(data1)


lst_so, lst_ta, lst_v,lst_clk= [],[],[],[]
for i in t.index:        
    so = se_1[se_1 == t.loc[i, "variable"]].index[0]
    ta = se_2[se_2 == t.loc[i, "Country"]].index[0]      
    v = abs(t.loc[i, "value"])
    clk = df_cl.loc[df_cl["node"] == t.loc[i, "variable"],
           "color_shadow"].values[0]
    lst_so.append(so)        
    lst_ta.append(ta)      
    lst_v.append(v)        
    lst_clk.append(clk) 
    
    
f = [0.18,0.4,0.6,0.9]
r = np.linspace(0.01, 0.99, len(lst_se1)).tolist()

mynode = dict(
      pad = 15,  thickness = 15, color = lst_cl+['#c4c4c4']*len(lst_se1),
      line = dict(color = "black", width = 0.5),
     label = se_1.tolist()+se_2.tolist(), 
      x = [0.01]*len(lst_i) +[0.15]*len(lst_se1), y = f+r)         ## to narrow the width (x) and save the space for extra annotations

fig = go.Figure(data=[go.Sankey(
  arrangement = "snap",
  node = mynode,
  link = dict(source = lst_so, target = lst_ta,                            ## indices correspond to labels, eg A1, A2, A1, B1, ...
              value = lst_v, color = lst_clk) )])
    
fig.show()
fig.write_image('./model/svg/sk_bar_metp.svg')
    